#ifndef __ucr_config_H
#define __ucr_config_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: config.h,v 1.3 1996/11/10 19:28:59 steve Exp $"
/*
 * $Log: config.h,v $
 * Revision 1.3  1996/11/10 19:28:59  steve
 *  Add setjmp and longjmp for the i386,
 *  Add support for linux system calls on i386,
 *  and add a LinuxIO device driver.
 *
 * Revision 1.2  1996/11/09 04:34:20  steve
 *  Add i386_linux thread infrastructure.
 *
 * Revision 1.1  1996/11/08 21:40:18  steve
 *  Divide up thread_create for easier porting, Add the
 *  sparc_linux and i386_linux targets, and adjust makefiles
 *  to allow for them. Document the new concept of virtual boards.
 *
 *  Change the linker output file extension to .exe, because
 *  some targets use ELF and not COFF.
 *
 *  Get rid of some trivial warnings.
 *
 */

/*
 * This configuration does not support ISRs (yet) so stub this out.
 */
#define isr_hot_flag() (0)

# include  <i386/ucr_asm.h>

/*
 * This configuration supports calls to the host, the linux
 * kernel. For the i386 CPU, the system call is an int $0x80 with the
 * %eax register filled with the system call number. The result is
 * returned to the same register.
 *
 * Let the compiler figure out how to get the values into the
 * registers, I just say in the asm statements where the parameters
 * must be.
 */

inline int call_host0(int sys)
{
	/* The 'a' constraint is "only the %eax register" for i386. */
      asm __volatile__ ("int  $0x80\n" : "=a" (sys) : "0" (sys) );
      return sys;
}

inline int call_host1(int sys, int parm)
{
      asm __volatile__ ("int  $0x80\n"
			: "=a" (sys)
			: "0" (sys), "b" (parm) );
      return sys;
}

inline int call_host3(int sys, int parm1, int parm2, int parm3)
{
      asm __volatile__ ("int  $0x80\n"
			: "=a" (sys)
			: "0" (sys), "b" (parm1) , "c" (parm2), "d" (parm3));
      return sys;
}

/*
 * These are the system call numbers of interest.
 *
 * Why didn't I just include <asm/unistd.h> from the host environment?
 * Well, maybe I should but for now I err in favor of not including
 * the clutter of all the other stuff. In the end, I want the names of
 * these numbers to look the same on any target that supports them,
 * and besides the programmer should not be using linux stuff except
 * to get the test bench to work. I might yet change my mind and
 * include the linux header file, the day is young.
 */

# define __SYSCALL_exit  1
# define __SYSCALL_read  3
# define __SYSCALL_write 4
# define __SYSCALL_open  5
# define __SYSCALL_close 6

# endif
