
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *   Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 *
 * $Id: bootprom.ld,v 1.6 1998/04/03 01:35:12 steve Exp $
 *
 * $Log: bootprom.ld,v $
 * Revision 1.6  1998/04/03 01:35:12  steve
 *  Add assertion support.
 *
 * Revision 1.5  1997/09/04 07:37:54  steve
 *  Get rid of excessive _ in xTOR_TABLE symbols.
 *
 * Revision 1.4  1997/08/25 17:43:02  steve
 *  Allow memory layout be set by link models.
 *
 * Revision 1.3  1996/10/28 03:45:04  steve
 *  License notice.
 *
 * Revision 1.2  1996/10/11 00:23:17  steve
 *  Rearrange the memory map to reflect the theory of
 *  operation for the ISE board.
 *
 * Revision 1.1  1996/10/08 03:28:47  steve
 *  Add the ISE bootstrap prom.
 *
 */

/*
 * The memory map for the linked result places the text and IBR in the
 * flash memory, and the data and bss sections into the dram. The image
 * is delivered to the board by stuffing everything in the flash memory.
 * To get this affect, I use the AT() options for each output section to
 * LINK it in the ram or whatever but LOAD it into the given location
 * relative to the beginning of flash memory.
 *
 * When the flash memory is programmed and stuffed on the board, the
 * .text and IBR sections are at the right address. The startup code is
 * then smart enough to copy the .data image from the flash into DRAM.
 *
 * Note that I restrict myself to the high 1Meg of DRAM because the
 * program that runs out of flash is expected to load a real program
 * into DRAM. By using only a little bit of the end of DRAM, I can use
 * beginning for the sub-program's text and data, and then upon reset
 * the DRAM that I've been using goes to the heap of the child.
 */

MEMORY
{
	/* Put DRAM in region 2_3. */
	dram  : org = 0xa0000000 + 3M, len = 1M

	  /* FLASH memory is fixed in hardware to be 512K bytes,
	     and we want it at the end so we can put the IMI there,
	     especially the initial boot record. */
	flash : org = 0xff000000 - 512K, len = 512K
	IBR   : org = 0xfeffff30,  len = 208
}

	  /* The ISE board has a couple little registers for controlling
	     the few non-PCI devices on the local bus. */

__ISE_DRAM    = 0xa0000000;
__ISE_FLASH   = 0xfef80000;
__ISE_STATUS  = 0xb0000000;
__ISE_CONTROL = 0xb0000000;

__index_space__ = 0xa0000000;

ENTRY(start)
STARTUP(locore.o)

SECTIONS
{
	  /* Executable code, constants, and the constructor table
	     go in the beginning of the flash memory. */

	.text : AT(0) {
		*(.text)
		. = ALIGN(4);
		__CTOR_TABLE__ = .;
		*(.ctors)
		LONG(0);
		__DTOR_TABLE__ = .;
		*(.dtors)
		LONG(0);
		_etext = .;
	} > flash


	  /* Initialized data goes into the DRAM. Load load an image
	     of this section in the FLASH after the .text section so
	     the startup code can find it and copy it in place. */

	.data : AT( SIZEOF(.text) ) {
		. = ALIGN(16); _bdata = .;
		*(.data)
		*(COMMON)
		_edata = .;
	} > dram


	  /* The .bss section is uninitialized data. I need to link
	     to it, but it doesn't get loaded. */
	.bss (NOLOAD) : {
		*(.bss)
		_end = .;
	}  >dram

	_end_heap = _bdata + 1M;


	  /* The IBR is special and magical. It must be in a very
	     specific place, so placing it there is the job of the
	     linker, that's me. It is loaded into the end of flash
	     memory. */

	ibr : AT (ADDR(ibr) - ADDR(.text)) {
		*(ibr)
	} > IBR


}
ibr_checksum = -(start + prcb);
