#ifndef __ucr_assert_H
#define __ucr_assert_H


/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: assert.h,v 1.7 1998/05/26 16:02:49 steve Exp $"
/*
 * $Log: assert.h,v $
 * Revision 1.7  1998/05/26 16:02:49  steve
 *  Add panic.
 *
 * Revision 1.6  1998/04/11 23:41:52  steve
 *  Add cell extent tests.
 *
 * Revision 1.5  1997/08/31 21:27:48  steve
 *  proper assert statement syntax.
 *
 * Revision 1.4  1996/10/28 03:45:08  steve
 *  License notice.
 *
 * Revision 1.3  1996/08/08 20:57:20  steve
 *  Minor optimizations by inlining a few things
 *  and setting optimization attributes.
 *
 * Revision 1.2  1996/05/24 01:46:33  steve
 *  Initial port to i960_cyclone
 *  Seperate some files in the library to seperate files
 *  for making smaller binaries, use uCR_ prefix for private
 *  symbols, move assembly files around for compile-time
 *  portability, change __start to ucr_start and give it
 *  new behavior, and simplify the ucr_assert implementation.
 *
 * Revision 1.1  1995/10/18 06:11:33  steve
 *  Make a target specific include directory.
 *
 * Revision 1.1  1995/10/17 21:05:12  steve
 *  UCR specific assert.
 *
 */

/*
 * This header file is required by the unhosted ANSI C standard.
 *
 * Section numbers in comments below refer to the document:
 *	American National Standard for Information Systems --
 *		Programming Language C
 * dated February 14, 1990, or any compatible revision.
 */

#if defined(__cplusplus)
extern "C" {
#endif

/*
 * The assert() macro is as required by section 4.2.1.1. The ucr_assert
 * function is part of the ucr library unless overridden by the
 * application.
 *
 * The default behavior of a failed assert is to abort without
 * printing anything. The application can get more sophisticated
 * behavior out of the assert by supplying a more sophisticated
 * ucr_assert() function.
 */
#if defined(NDEBUG)
#define assert(ignore)
#define assert_mem_ext(x,y)
#else

extern void ucr_assert(const char*, unsigned) __attribute__ ((noreturn));
extern void ucr_panic(const char*, unsigned) __attribute__ ((noreturn));
extern int heap_test_ext(const void*memory, const void*ptr);

#define assert(expr) do { \
	if (!(expr)) ucr_assert(__FILE__,  __LINE__); } while(0)

#define assert_mem_ext(mem,ptr) do { \
     if (heap_test_ext((mem),(ptr))) ucr_assert(__FILE__,__LINE__); } while(0)

#endif

#if defined(__cplusplus)
}
#endif

#endif
