#ifndef __chips_LinuxIO_H
#define __chips_LinuxIO_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 */
#ident "$Id: LinuxIO.h,v 1.1 1996/11/10 19:29:00 steve Exp $"
/*
 * $Log: LinuxIO.h,v $
 * Revision 1.1  1996/11/10 19:29:00  steve
 *  Add setjmp and longjmp for the i386,
 *  Add support for linux system calls on i386,
 *  and add a LinuxIO device driver.
 *
 */

/*
 * When using a Linux process as a target, there are all kinds of I/O
 * opportunities that don't otherwise exist. This class provides the
 * perception of the Linux kernel as a kind of device with posix-like
 * operations.
 *
 * This class provides and stdio device type, and so can be used with
 * the stdio library. The device type string is "linux" and the
 * instance name is a file name, so the following are valid names:
 *
 *	linux:/tmp/output  <-- absolute path
 *	linux:test_vectors <-- relative path
 *      linux:stdin
 *	linux:stdout
 *	linux:stderr
 *
 * NOTE: This class is specifically intended to provide an interface
 * to Linux, and NOT to create a Linux environment for
 * applications. This pseudo-device only applies to linux process
 * targets.
 */

# include  <StdioDevice.h>

class LinuxIO : public StdioDeviceType {

    public:
      LinuxIO();
      ~LinuxIO();

    private:
      StdioDevice*open(const char*name);
      void close(StdioDevice*);

    private: // Not implemented
      LinuxIO(const LinuxIO&);
      LinuxIO& operator= (const LinuxIO&);
};


#endif
