#ifndef __gdb_gdb_if_H
#define __gdb_gdb_if_H
/*
 * Copyright (c) 1996-1998 Picture Elements, Inc.
 *    Stephen Williams <steve@picturel.com>
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: gdb_if.h,v 1.3 1998/07/16 22:50:35 steve Exp $"
/*
 * $Log: gdb_if.h,v $
 * Revision 1.3  1998/07/16 22:50:35  steve
 *  Watch out for 64bit hosts!
 *
 * Revision 1.2  1996/10/28 03:45:14  steve
 *  License notice.
 *
 * Revision 1.1  1996/08/28 00:40:31  steve
 *  Add the GDB proxy to uCR.
 *
 */

/*
 * Communication with the GDB proxy works with this packet header. The
 * debugger controlls the entire communication by sending
 * requests. The target replies at least with a gdb_if header. It
 * includes data after if data is forthcomming.
 */

#if defined(__alpha__)
typedef unsigned gdbif_u32_t;
#else
typedef unsigned long gdbif_u32_t;
#endif

struct gdb_if {
      gdbif_u32_t magic1;

	/* This sequence is used by the sender to make sure the reply
	   received realy matches with the request sent. */
      gdbif_u32_t sequence;

	/* This is the command to be executed or being responded
	   to. If this is a response, the ack member is filled with an
	   error code reflecting any errors. */
      unsigned char cmd_code;
      unsigned char ack;

	/* This is the number of 32bit words that follow the
	   header. This is not necessarily the same as the number of
	   interesting bytes in the argument. (Could be different by
	   3, for example.) */
      unsigned short ext;

	/* These are arguments to the command. The meanings depend on
	   the command code. */

      gdbif_u32_t arg1, arg2, arg3, arg4;

      gdbif_u32_t magic2;
};

/*
 * These magic numbers mark packets. They are an assertion check that
 * packet headers really are where I expect them to be. In order to be
 * easy to find in core dumps, the magic numbers are, in ascii:
 *	GDB!
 *	gdb?
 * The letters may go the other way around on little-endian machines.
 */
# define GDB_IF_MAGIC1 0x47444121
# define GDB_IF_MAGIC2 0x6764623f

/*
 * The ack field is set to 0 by the sender, and set to one of these
 * values when the reply is sent:
 */
# define GIF_ACK_OK 0
# define GIF_ACK_NOSYS 1

/*
 * This does nothing. It is useful for pinging that the target is
 * responding to debugger commands, without doing anything major.
 */
# define GIF_CMD_NOOP 0

/*
 * The debugger writes bytes into the target memory. args[0] is the
 * address where the bytes go, and args[1] is the number of bytes. The
 * actual values follow the protocol packet. The read command reads
 * the bytes back out. The target includes the bytes after the
 * response packet.
 */
# define GIF_CMD_WRITE 1
# define GIF_CMD_READ 2

/*
 * Get a list of threads that currently exist. The return is a list of
 * the threads by THREAD id, an opaque number understood by uCR. This
 * function assumes that the args are 0.
 */
# define GIF_CMD_THREADS 3

/*
 * Get or set some subset of the available registers. This registers
 * are numbered the same way GBD orders them in the config/ARCH/tm-cpu.h
 * header file. This is different for each type of CPU.
 *
 * arg1 has the tid of the thread to affect.
 *
 * For SET_REGS, arg2 has the number of the register to affect, and
 * arg3 the value to write to the register.
 */
# define GIF_CMD_GET_REGS 4
# define GIF_CMD_SET_REGS 5

/*
 * Test the presence of a thread, and return its state. This is
 * generally used to verify that a thread really exists.
 *
 * arg1 contains the thread to be tested, and in the response contains
 * one of the following code numbers.
 *		0 -- thread does not exist
 *		1 -- thread exists (subject to change.)
 */
# define GIF_CMD_THREAD_TEST 6

/*
 * This command returns the id of the next faulty thread in arg1, and
 * the code that caused the fault in arg2. There there are no faulty
 * threads, then the reply is withheld until one appears. This is a
 * lot like the unix wait(2) system call.
 *
 * If arg1 is 1 (as opposed to 0) then WAIT will not wait for a fault
 * to happen and will instead suspend the application no matter
 * what. If there are no faulty threads, then arg1 is set to an
 * arbitrary thread and arg2 set to ULONG_MAX.
 */
# define GIF_CMD_WAIT 7

/*
 * RESUME allows the debugger to resume a specific thread, or all the
 * threads that are not faulty. If a thread is faulty, then resuming
 * it explicitly will cause the thread to be moved to the ready list
 * where a generic resume will cause the thread to be resumed.
 *
 * arg1 is 0 to resume all the threads, or
 * non-0 to resume a specific thread.
 */
# define GIF_CMD_RESUME 8

/*
 * Insert/remove a breakpoint at a given location. When setting a
 * breakpoint, GDB writes the mark instruction into the location in
 * arg1 (the breakpoint address) and returns the bytes that were
 * replaced. The debugger must store the memory and return it with the
 * CLR command when removing the breakpoint. GDB is expected to know
 * what the breakpoint instruction is.
 *
 * When the processor hits the breakpoint, GDB will fix up the
 * instruction pointer so that it points to the instruction about to
 * be executed.
 */
# define GIF_CMD_SET_BREAK 9
# define GIF_CMD_CLR_BREAK 10

#endif
