#ifndef __i960_ucr_asm_H
#define __i960_ucr_asm_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ucr_asm.h,v 1.2 1996/10/28 03:45:16 steve Exp $"
/*
 * $Log: ucr_asm.h,v $
 * Revision 1.2  1996/10/28 03:45:16  steve
 *  License notice.
 *
 * Revision 1.1  1996/09/19 04:43:52  steve
 *  Add the i960_ise target, change the target
 *  configuration headers to allow sharing by
 *  targets, fix up the make clean rule.
 *
 * Revision 1.3  1996/08/24 20:57:17  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.2  1996/08/10 04:04:36  steve
 *  Inline the thread_spl things where possible,
 *  stop basing heap on estack, and fix startup
 *  code to use end_heap.
 *
 * Revision 1.1  1996/08/03 03:30:34  steve
 *  Create the ucr/asm.h header as a target specific
 *  header with line functions, etc; Replace the
 *  __isr_run_flag with a bit of inline assembly code.
 *
 *  Move much thread code out of the __Secheduler class
 *  and into more public thread_ functions, to eliminate
 *  some wasted function call depth. Even the main_thread
 *  is move into its own file.
 *
 *  Keep a list of available threads for debug use, and
 *  add a function to get thread registers.
 *
 */

/* NOTE:
 * You should be getting at this header file by including
 *	<ucr/config.h>
 * This is a header file put in the target specific include directory
 * and includes the header for the right CPU type, like this one if
 * you are using an i960.
 */

/* NOTE:
 * This header file may be included by C or C++ alike. I need to make
 * sure things are parseable by either when implementing this.
 */

/*
 * This function returns true if the processor is running in
 * interrupted state, false otherwise.
 */
#if defined(__cplusplus)
inline bool isr_hot_flag()
#else
inline int  isr_hot_flag()
#endif
{
      register unsigned tmp;
      asm("modpc 0, 0, %0" : "=r" (tmp));
      return tmp & 0x2000;
}

/*
 * The spl cookie has the bits of the PC registers that correspond to
 * the processor priority (PC.p) and the trace enable bit
 * (PC.te). Other bits may be on or off, but will be ignored.
 */
typedef unsigned long spl_cookie_t;

/*
 * Return the current interrupt priority from the process controlls
 * register, replacing the current value with 31. This little bit of
 * magic is performed with the modpc instruction.
 */
inline spl_cookie_t thread_splhi()
{
      spl_cookie_t tmp = 0x001f0000;
      const unsigned long mask = 0x001f0001;

      asm volatile ("modpc %2,%2,%0"
		    : "=r" (tmp)
		    : "0" (tmp), "r" (mask));
      return tmp;
}

/*
 * Use the modpc instruction to simultaneously restore the processor
 * priority from the cookie and return the current priority.
 */
inline spl_cookie_t thread_spl(spl_cookie_t cookie)
{
      const unsigned long mask = 0x001f0001;
      asm volatile ("modpc %2, %2, %0"
		    : "=r" (cookie)
		    : "0" (cookie), "r" (mask));
      return cookie;
}

#if defined(__cplusplus)
extern "C" void thread_spl_low();
#else
extern void thread_spl_low();
#endif

#endif
