#ifndef __ucr_string_H
#define __ucr_string_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: string.h,v 1.6 1998/06/30 00:00:57 steve Exp $"
/*
 * $Log: string.h,v $
 * Revision 1.6  1998/06/30 00:00:57  steve
 *  Add memchr.
 *
 * Revision 1.5  1997/07/31 22:47:58  steve
 *  Add more ISO-C functions.
 *
 * Revision 1.4  1997/04/25 18:45:51  steve
 *  More ISO C support.
 *
 * Revision 1.3  1996/10/28 03:45:10  steve
 *  License notice.
 *
 * Revision 1.2  1996/09/12 21:46:42  steve
 *  Add more string functions.
 *
 * Revision 1.1  1996/07/30 03:18:07  steve
 *  Add the string.h header, and strlen.
 *
 */

# include  <stddef.h>

/*
 * This header file is required by the hosted ANSI C standard. It
 * contains types and prototypes for standard functions. Not
 * everything that the standard requires of this file is supported by
 * the environment.
 *
 * Section numbers in comments below refer to the document:
 *	American National Standard for Information Systems --
 *		Programming Language C
 * dated February 14, 1990, or any compatible revision.
 */

#if defined(__cplusplus)
extern "C" {
#endif

extern void* memcpy(void*s1, const void*s2, size_t n);
extern void* memset(void*s1, int c, size_t n);
extern void* memchr(const void*s, int c, size_t n);
extern int memcmp(const void*s1, const void*s2, size_t n);

/* Section 4.11.2.3
 * Copy s2 to s1, including the terminating null.
 */
extern char*strcpy(char*s1, const char*s2);

/* Section 4.11.2.4
 * Copy n characters from s2 to s1. Pad s1 with nulls if necessary.
 */
extern char*strncpy(char*s1, const char*s2, size_t n);

/* Section 7.11.4.2
 * Compare 2 strings.
 */
extern int strcmp(const char*s1, const char*s2);
extern int strncmp(const char*s1, const char*s2, size_t n);


/* Section 7.11.5.2
 * Return a pointer to the first occurence of c in the string.
 */
extern char*strchr(const char*s, int c);

/* Section 7.11.5.3
 *
 */
extern size_t strcspn(const char*s1, const char*s2);

/* Section 7.11.5.4
 * Return pointer to first instance of a character in s2 in s1.
 */
extern char* strpbrk(const char*s1, const char*s2);

/* Section 7.11.5.5
 * Return pointer to the last c in s1.
 */
extern char*strrchr(const char*s, int c);

/* Section 7.11.5.7
 * Return pointer to s2 in s1.
 */
extern char*strstr(const char*s1, const char*s2);

/* Section 4.11.6.3
 * The strlen function computes the length of the string pointed to by
 * s. */
extern size_t strlen(const char*s);

#if defined(__cplusplus)
}
#endif

#endif
