#ifndef __ucr_IntrList_H
#define __ucr_intrList_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: IntrList.h,v 1.3 1996/10/28 03:45:22 steve Exp $"
/*
 * $Log: IntrList.h,v $
 * Revision 1.3  1996/10/28 03:45:22  steve
 *  License notice.
 *
 * Revision 1.2  1995/10/20 04:02:40  steve
 *  Make timeouts lighter, less buggy, and a
 *  little less general. Light is better here.
 *  Also, leave lists as a general tool.
 *
 * Revision 1.1  1995/09/17 07:00:28  steve
 *  Add support for timeouts.
 *
 */


/*
 * For intrusive lists, the type T must have a member next_ that is
 * accessible to IntrList and points to a T. For example, to make a
 * list of integers, create a class like this:
 *
 * class intCell {
 *    public:
 *      int value;
 *    private:
 *      friend class IntrList<intCell>;
 *      intCell*next_;
 * };
 */

template <class T> class IntrList {

    public:
      IntrList() : last_(0) { }

      void append(T*cell)
	{
	      if (last_) {
		    cell->next_ = last_->next_;
		    last_->next_ = cell;
	      } else {
		    cell->next_ = cell;
	      }
	      last_ = cell;
	}

      void prepend(T*cell)
	{
	      if (last_) {
		    cell->next_ = last_->next_;
		    last_->next_ = cell;
	      } else {
		    cell->next_ = cell;
		    last_ = cell;
	      }
	}

      T* pull()
	{
	      T* result = last_;
	      if (result) {
		    last_ = result->next_;
		    if (last_ == result) last_ = 0;
	      }
	      return result;
	}

    private:
      T* last_;

    private: // not implemented
      IntrList(const IntrList<T>&);
      IntrList<T>& operator= (const IntrList<T>&);
};



#endif
