#ifndef __ucr_Mutex_H
#define __ucr_Mutex_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: Mutex.h,v 1.2 1996/10/28 03:45:23 steve Exp $"
/*
 * $Log: Mutex.h,v $
 * Revision 1.2  1996/10/28 03:45:23  steve
 *  License notice.
 *
 * Revision 1.1  1995/09/02 18:27:10  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */


/*
 * Provide the class mutex and condition variable types. The two are
 * meant to be used together, although one may use mutexes without
 * condition variables.
 *
 * NOTE: An ISR may interrupt a thread holding a mutex. If so, an
 * attempt by the ISR to enter the mutex will deadlock. Thus, Mutexes
 * and conditions are not suitable for ISR use.
 */

# include  <ucr/ThreadQueue.h>

class Condition;
class Mutex;


class Mutex {

      friend class Condition;

    public:
      Mutex();
      ~Mutex();

    public:
      void enter();
      void leave();

    private:
      unsigned char busy_flag;
      ThreadQueue  stuck_;

    private: // not implemented
      Mutex(const Mutex&);
      Mutex& operator= (const Mutex&);
};

/*
 * The Condition methods can only be called by threads that are in the
 * mutex.
 */
class Condition {

    public:
      explicit Condition(Mutex&);
      ~Condition();

    public:
      void wait();
      void signal();
      void broadcast();

    private:
      ThreadQueue waiting_;

    private:
      Mutex& master_;

    private: // not implemented
      Condition(const Condition&);
      Condition& operator= (const Condition&);
};

#endif
