#ifndef __ucr_RingBuffer_H
#define __ucr_RingBuffer_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: RingBuffer.h,v 1.4 1997/08/11 19:58:11 steve Exp $"
/*
 * $Log: RingBuffer.h,v $
 * Revision 1.4  1997/08/11 19:58:11  steve
 *  Add the SyncRing class.
 *
 * Revision 1.3  1996/10/28 03:45:23  steve
 *  License notice.
 *
 * Revision 1.2  1996/07/24 05:40:48  steve
 *  Add the peek method.
 *
 */


/*
 * This template implements a simple ring buffer. The size is the
 * number of objects that can be stored, and the type T is the type of
 * the objects that can be store. T must support default
 * initialization (to construct the buffer array), and assignment. The
 * RingBuffer will copy values in and out.
 */
template <class T, unsigned size> class RingBuffer {

    public:
      RingBuffer() : count_(0), inp_(0), outp_(0) { }

      unsigned count() const { return count_; }
      unsigned limit() const { return size; }
      unsigned space() const { return size - count_; }

      void put(T value)
	{
	      if (count_ == size) return;

	      values_[inp_++] = value;
	      inp_ %= size;
	      count_ += 1;
	}

      T get()
	{
	      if (count_ == 0) return T();
	      unsigned idx = outp_++;
	      outp_ %= size;
	      count_ -= 1;
	      return values_[idx];
	}

      T& peek()
        { return values_[outp_]; }

    private:
      unsigned count_;
      unsigned inp_, outp_;

      T values_[size];

};

#endif
