/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: I960RP.cc,v 1.11 1998/05/30 18:39:04 steve Exp $"

# include  <chips/I960RP.h>
# include  <ucr/ucr.h>
# include  <ucr/param.h>
# include  <assert.h>

/*
 * The location of various registers in the I960RP are fixed by the
 * chip design. Therefore, I do not take a base address as a
 * parameter, and I use constants at pointers.
 */

struct ATU_REGS {
      unsigned short pad00;
      unsigned short pad02;
      unsigned short PATUCR; // Primary ATU Command Register
      unsigned short PATUSR; // Primary ATU Status Register

      unsigned char  ATURID; // ATU Revision ID Register
      unsigned char  ATUCCR[3]; // ATU Class Code Register
      unsigned char  ATUCLSR;   // ATU Cacheline Size Register
      unsigned char  ATULT;     // ATU Latency Timer Register
      unsigned char  ATUHTR;    // ATU Header Type Register
      unsigned char  ATUBISTR;  // ATU BIST Register
	// 0x10
      unsigned char  pad10[16];
	// 0x20
      unsigned char  pad20[16];
	// 0x30
      unsigned char  pad30[16];
	// 0x40
      unsigned char  pad40[16];
	// 0x50
      unsigned char  pad50[16];
	// 0x60
      unsigned char  pad60[16];
	// 0x70
      unsigned char  pad70[16];
	// 0x80
      unsigned char  pad80[8];
      unsigned long  ATUCR; // ATU Configuration Register
      unsigned long  pad8c;
	// 0x90
      unsigned long PATUISR; // Primary ATU Interrupt Status Register
      unsigned long SATUISR; // Secondary ATU Interrupt Status Register
};

/*
 * The names used in this register layout match the names used
 * by Intel in the i960 RP Microprocessor User's manual,
 * Table 17-4
 */
struct MU_REGS {
      unsigned long ARSR; // APIC Register Select Register
      unsigned long pad04_;
      unsigned long AWR;  // APIC Window Register
      unsigned long pad0c_;
      unsigned long IMR[2]; // Inbound Message Register 0/1
      unsigned long OMR[2]; // Outbound Message Register 0/1
      unsigned long IDR;  // Inbound Doorbell Register
      unsigned long IISR; // Inbound Interrupt Status Register
      unsigned long IIMR; // Inbound Interrupt Mask Registers
      unsigned long ODR;  // Outbound Doorbell Register
      unsigned long OISR; // Outbound Interrupt Status Register
      unsigned long OIMR; // Outbound Interrupt Mask Register
      unsigned long pad38_;
      unsigned long pad3c_;
      unsigned long pad40_;
      unsigned long pad44_;
      unsigned long pad48_;
      unsigned long pad4c_;
      unsigned long MUCR; // Messaging Unit Configuration Register
      unsigned long QBAR; // Queue Base Address Register
      unsigned long pad58_;
      unsigned long pad5c_;
      unsigned long IFHPR;// Inbound Free Head Pointer Register
      unsigned long IFTPR;// Inbound Free Tail Pointer Register
      unsigned long IPHPR;// Inbound Post Head Pointer Register
      unsigned long IPTPR;// Inbound Post Tail Pointer Register
      unsigned long OFHPR;// Outbound Free Head Pointer Register
      unsigned long OFTPR;// Outbound Free Tail Pointer Register
      unsigned long OPHPR;// Outbound Post Head Pointer Register
      unsigned long OPTPR;// Outbound Post Tail Pointer Register
      unsigned long IAR;  // Index Address Register
};

/*
 * The DMA Chain descriptor cell is described in 20.3.1 of the i960 RP
 * Microprocessor User's Manual.
 */

struct DMA_CELL {
      DMA_CELL*NDA;     // Next Descriptor Address
      unsigned long PAD;  // PCI Address
      unsigned long PUAD; // PCI Upper Address
      void* LAD;        // 80960 local address
      unsigned long BC; // Byte Count
      unsigned long DC; // Descriptor Control

      unsigned long pad_[2];

      void* operator new (size_t);
      void operator delete(void*);
    private:
      static DMA_CELL*list_;
};

/*
 * Use this cell for the single-chain-cell DMA operations that I
 * do. Note that the alignment constraints are rather severe.
 */
extern DMA_CELL dma_cell_heap[6] asm ("$dma_cell");
static DMA_CELL*make_dma_cell_heap()
{
      DMA_CELL*list = 0;
      for (unsigned idx = 0 ; idx < 6 ;  idx += 1) {
	    DMA_CELL*cur = dma_cell_heap + idx;
	    cur->NDA = list;
	    list = cur;
      }

      return list;
}

DMA_CELL*DMA_CELL::list_ = make_dma_cell_heap();

void* DMA_CELL::operator new(size_t size)
{
      assert( size == sizeof(DMA_CELL) );
      assert( list_ );
      DMA_CELL*result = list_;
      list_ = result->NDA;
      return result;
}

void DMA_CELL::operator delete(void*p)
{
      if (p == 0) return;
      DMA_CELL*cur = (DMA_CELL*)p;
      cur->NDA = list_;
      list_ = cur;
}

static DMA_CELL* pending_ptr[3] = { 0, 0, 0 };


struct DMA_CHANNEL {
      volatile unsigned long CCR;   // Channel Control Register
      volatile unsigned long CSR;   // Channel Status Register
      unsigned long pad08;
      volatile DMA_CELL* DAR; // Descriptor Addres Register
      DMA_CELL*NDAR;          // Next Descriptor Address
      unsigned long PADR;  // PCI Address
      unsigned long PUADR; // PCI Upper Address
      void* LADR;          // 80960 local address
      unsigned long BCR;   // Byte Count
      unsigned long DCR;   // Descriptor Control
};

static volatile ATU_REGS*const ATU = (volatile ATU_REGS*) 0x1200;
static volatile MU_REGS*const  MU  = (volatile MU_REGS*)  0x1300;

static unsigned short&ATUVID = *(unsigned short*)0x1200;
static unsigned short&ATUDID = *(unsigned short*)0x1202;
static volatile unsigned long&POMWVR = *(volatile unsigned long*)0x1254;

static unsigned long&IIMR = *(unsigned long*)0x1328;
static unsigned long&OIMR = *(unsigned long*)0x1334;

static volatile DMA_CHANNEL*const DMA[3] = {
      (DMA_CHANNEL*)0x1400,
      (DMA_CHANNEL*)0x1440,
      (DMA_CHANNEL*)0x1480
};


I960RP::I960RP()
: bist_sync_(false)
{
      ATUVID = 0x12c5;
      ATUDID = 0x007f;

	// Turn off BIST bits
      ATU->ATUBISTR = 0;

	// Disable the BIST, enable primary translation
      ATU->ATUCR = 0x02;

      set_isr(XINT7, mu_isr_, this);
      set_isr(XINT6, dma_isr_, this);

	// Turn off the circular queue subsystem and clear message
	// registers.
      MU->MUCR = 0x00000002;

	// Clear interrupt.
      MU->IDR = 0xffffffff;
      MU->IISR= 0x000001ff;

	// Enable only the inbound doorbell interrupts, and disable
	// the outbound message box interrupts.
      IIMR = 0x1fb;
      OIMR = 0x0fb;

	// Disable all the DMA channels
      DMA[0]->CCR = 0;
      DMA[1]->CCR = 0;
      DMA[2]->CCR = 0;
}

I960RP::~I960RP()
{
	// Turn off the BIST
      ATU->ATUBISTR = 0;

	// Disable the BIST, and all the other ATU actions.
      ATU->ATUCR = 0;

	// Disable all the DMA channels
      DMA[0]->CCR = 0;
      DMA[1]->CCR = 0;
      DMA[2]->CCR = 0;
}

void I960RP::ring_host(unsigned idx)
{
      assert(idx < 28);
      MU->ODR = 1 << idx;
}

/*
 * Turning on the BIST turns on the BIST Capable flag in the PCI
 * config space. The completion code and start bit are in this case
 * turned off. Disabling the BIST feature leaves the other bits unchanged.
 */
void I960RP::bist_enable(bool flag)
{
      if (flag) { ATU->ATUCR |= 0x08; ATU->ATUBISTR = 0x80; }
      else ATU->ATUCR &= ~0x08;
}

/*
 * This sets the BIST completion code and turns off the BIST start
 * bit. It leaves the BIST Capable flag untouched.
 */
void I960RP::bist_finished(unsigned char code)
{
      ATU->ATUBISTR = (ATU->ATUBISTR & 0x80) | (code & 0x0f);
}

/*
 * The bist_sync_ lock is initialized locked. When a BIST Start
 * interrupt arrives, the lock is released and a wait() will
 * succeed. Only one wait will succeed, so only one thread does the
 * self test. The bist_finished() method above allows a new interrupt
 * to arrive, which allows for more locks to be released.
 */
void I960RP::bist_wait() const
{
      bist_sync_.get();
}

/*
 * In the I960RP interface, the input mailboxes are assigned to
 * inbound message registers and the output mailboxes are assigned to
 * outbound message registers. There are two of each.
 */
unsigned long I960RP::get_mailbox(unsigned idx) const
{
      assert( idx < 2 );
      return MU->IMR[idx];
}

void I960RP::set_mailbox(unsigned idx, unsigned long value)
{
      idx -= 2;
      assert( idx < 2 );
      MU->OMR[idx] = value;
}

/*
 * This method allows one transfer to be outstanding and another to be
 * ready. This allows the caller to use two buffers and not bother
 * with the dma_done() method until the very end. However, we can
 * completely fill in the next transfer cell before blocking. This may
 * increase overlap just a bit.
 */
void I960RP::dma_trans(unsigned ch, unsigned long pci, void*local,
		       unsigned long count, bool wr_flag)
{
      assert( count <= 0x00ffffff );
      assert( ch < 3 );

      DMA_CELL*cell = new DMA_CELL;
      cell->NDA = 0;
      cell->PAD = pci;
      cell->PUAD = 0;
      cell->LAD = local;
      cell->BC  = count;

	// Use Memory Write command for writes to the PCI bus, and
	// Memory Read commands for reads from the PCI bus.
      if (wr_flag) {
	    cell->DC = 0x17;
      } else if (ch == 2) {
	      // XXXX For now, the secondary bus is assumed to be some
	      // fixed cacheline size.
	    const unsigned long CACHE_LINE = 4; // units of dwords
	    const unsigned long CACHE_MASK = CACHE_LINE*4 - 1;

	    unsigned long lines = (pci + count) & ~CACHE_MASK;
	    lines -= pci & ~CACHE_MASK;
	    switch (lines / CACHE_LINE) {
		case 0: // Ordinary read if I cross no cache boundries
		  cell->DC = 0x16;
		  break;
		case 1: // Memory Read Line
		  cell->DC = 0x1e;
		  break;
		default: // Memory Read Multiple
		  cell->DC = 0x1c;
		  break;
	    }

      } else {
	      // XXXX Cheat and not use MRL or MRM on the primary bus,
	      // XXXX just ordinary Memory Read commands.
	    cell->DC = 0x16;
      }

	// Write a pointer to the cell into channel X, then start the
	// operation with a channel enable bit.

      dma_sync_[ch].get();

      delete pending_ptr[ch];
      pending_ptr[ch] = cell;
      DMA[ch]->NDAR = cell;
      DMA[ch]->CCR = 1;
}

void I960RP::dma_done(unsigned ch)
{
      assert( ch < 3 );
      dma_sync_[ch].wait();
      delete pending_ptr[ch];
      pending_ptr[ch] = 0;
}

unsigned long I960RP::readl(unsigned long off)
{
      POMWVR = off & ~0x03ffffff;
      return *reinterpret_cast<unsigned long*>(0x80000000 | (off&0x03ffffff));
}

void I960RP::writel(unsigned long off, unsigned long val)
{
      POMWVR = off & ~0x03ffffff;
      *reinterpret_cast<unsigned long*>(0x80000000 | (off&0x03ffffff)) = val;
}


void I960RP::bus_transfer(unsigned long pci, void*local,
			  unsigned long count, bool wr_flag)
{
      dma_trans(1, pci, local, count, wr_flag);
}

/*
 * Return when the DMA1 channel is not busy.
 */
void I960RP::bus_transfer_done()
{
      dma_done(1);
}


void I960RP::mu_isr_(I960RP*obj, unsigned)
{
      unsigned long mask;

	// If there is an inbound doorbell interrupt (mask the NMI
	// interrupt) the call the BUS::Device method that handles
	// doorbells.
      mask = MU->IDR & 0x7fffffff;
      if (mask) {
	    MU->IDR = mask;
	    obj->bus_bell_isr(mask);
      }

      mask = ATU->PATUISR;
      if (mask) {
	    ATU->PATUISR = mask;

	    if (mask & 0x100) { // ATU BIST Interrupt
		  obj->bist_sync_.put();
	    }
      }
}

void I960RP::dma_isr_(I960RP*obj, unsigned)
{
      for (unsigned idx = 0 ;  idx < 3 ;  idx += 1)

	    if (obj->dma_sync_[idx].got()) {
		    // Get a mask of the interrupt state, and clear the bits
		    // that generated said interrupt.
		  unsigned long mask = DMA[idx]->CSR;
		  DMA[idx]->CSR = mask;

		  if (mask & 0x100) { // IF End of Chain Interrupt
			obj->dma_sync_[idx].put();
		  }

	    }
}

/*
 * $Log: I960RP.cc,v $
 * Revision 1.11  1998/05/30 18:39:04  steve
 *  Better reset behavior.
 *
 * Revision 1.10  1998/05/26 16:16:33  steve
 *  New channel protocol
 *
 */
