/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: M29F040.cc,v 1.1 1998/01/01 20:44:31 steve Exp $"


# include  <chips/M29F040.h>
# include  <assert.h>
# include  <ucr/ucr.h>

M29F040::M29F040(volatile void*base)
: base_((volatile unsigned char*)base)
{
}

unsigned char M29F040::manufacturer() const
{
      base_[0x5555] = 0xaa;
      base_[0x2aaa] = 0x55;
      base_[0x5555] = 0x90;
      unsigned char result = base_[0];
      base_[0] = 0xf0;
      return result;
}

unsigned char M29F040::device_code() const
{
      base_[0x5555] = 0xaa;
      base_[0x2aaa] = 0x55;
      base_[0x5555] = 0x90;
      unsigned char result = base_[1];
      base_[0] = 0xf0;
      return result;
}

bool M29F040::block_protected(unsigned long addr) const
{
      base_[0x5555] = 0xaa;
      base_[0x2aaa] = 0x55;
      base_[0x5555] = 0x90;
      unsigned char result = base_[(addr&0x30000) | 2];
      base_[0] = 0xf0;

      return result;
}

void M29F040::erase_block(unsigned long addr)
{
      assert( 0 == (addr & (BLOCK_SIZE-1)) );

	// Send the block erase command...
      base_[0x5555] = 0xaa;
      base_[0x2aaa] = 0x55;
      base_[0x5555] = 0x80;
      base_[0x5555] = 0xaa;
      base_[0x2aaa] = 0x55;
      base_[addr]   = 0x30;

	// Wait for the erase to complete.
      unsigned char prev = base_[addr] & 0x40;
      unsigned char next = base_[addr] & 0x40;
      while (prev != next) {
	    prev = next;
	    next = base_[addr] & 0x40;
	    thread_yield();
      }

	// Reset. The command is done.
      base_[0] = 0xf0;
}

void M29F040::program_bytes(unsigned long addr, const unsigned char*byte,
			    size_t cnt)
{
      for (unsigned idx = 0 ;  idx < cnt ;  idx += 1) {
	    base_[0x5555]   = 0xaa;
	    base_[0x2aaa]   = 0x55;
	    base_[0x5555]   = 0xa0;
	    base_[addr+idx] = byte[idx];

	      // Wait for the write to complete.
	    while ((byte[idx] & 0x80) != (base_[addr+idx] & 0x80))
		  ; /* busy wait */

      }

	// Reset. The command is done.
      base_[0] = 0xf0;
}

/*
 * $Log: M29F040.cc,v $
 * Revision 1.1  1998/01/01 20:44:31  steve
 *  Add flash driver
 *
 */

