#ifndef __chips_Region_H
#define __chips_Region_H
/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: region.h,v 1.1 1997/04/19 05:48:29 steve Exp $"
/*
 * $Log: region.h,v $
 * Revision 1.1  1997/04/19 05:48:29  steve
 *  Redo the BIOS allocator to be simpler and to
 *  properly handle holes.
 *
 */

/*
 * This class is designed to allocate aligned regions from an area
 * that is itself aligned. The size of the requested region is
 * expressed as an address mask, where the number of zeros in the low
 * bits is the requested size. For example, 0xfffffff0 is a size of 16
 * and 0xffff0000 is a size of 64K.
 */
class Region {

    public:
      Region(unsigned long addr, unsigned long mask);

	// Subtract a specific secion of memory from the region. This
	// can be used to create allocation holes.
      bool subtract(unsigned long addr, unsigned long mask);

	// Allocate a region, return the address.
      unsigned long allocate(unsigned long mask);

    private:
      struct CELL_ { unsigned long addr, mask; };

      enum { MAX_CELLS = 32 };
      CELL_ cells_[MAX_CELLS];
      unsigned ncells_;

      void split_(unsigned idx);
      void remove_(unsigned idx);

};


#endif
