
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ParallelPort_id.cc,v 1.3 1996/10/28 03:45:53 steve Exp $"
/*
 * $Log: ParallelPort_id.cc,v $
 * Revision 1.3  1996/10/28 03:45:53  steve
 *  License notice.
 *
 * Revision 1.2  1995/11/02 07:30:23  steve
 *  More convenient interface to id string.
 *
 * Revision 1.1  1995/11/02 06:57:18  steve
 *  Split up the ParallelPort source,
 *  Support nibble mode id strings.
 *
 */

/*
 * These methods support the reverse transfer of the device ID string
 * to the host. These are different from the normal reverse transfer
 * code because it does not need to be fast, and should not affect any
 * threads at all.  And so, it is entirely interrupt driven.
 */

# include  <mc68322/ParallelPort.h>
# include  "ParallelPort_priv.h"
# include  <ucr/ucr.h>
# include  <assert.h>

void ParallelPort::enter_nibble_id_mode_()
{
      nibble_sel_ = LOWS1;
      curid_ = id_;
      port_isr_ = port_isr_nibble_id_;
      port_intr_->enable_flags = LOW_nSelectIn|HIGH_nAutoFd|LOW_nAutoFd;
}

void ParallelPort::port_isr_nibble_id_()
{
      unsigned short lines = port_regs_->lines;
      unsigned char nibble;

      const unsigned short N_MASK = PIN_nSelectIn|PIN_nAutoFd|PIN_nAck2;
      switch (lines&N_MASK) {

	      // Event 7: Host lowers nAutoFd
	  case PIN_nSelectIn:
	    if (curid_ >= id_+nid_) break;

	    switch (nibble_sel_) {
		case LOW:
		  nibble = *curid_;
		  nibble_sel_ = HIGH;
		  break;
		case HIGH:
		  nibble = *curid_ >> 4;
		  nibble_sel_ = LOW;
		  curid_ += 1;
		  break;
		case LOWS1:
		  nibble = nid_ >> 8;
		  nibble_sel_ = HIGHS1;
		  break;
		case HIGHS1:
		  nibble = nid_ >> 12;
		  nibble_sel_ = LOWS2;
		  break;
		case LOWS2:
		  nibble = nid_;
		  nibble_sel_ = HIGHS2;
		  break;
		case HIGHS2:
		  nibble = nid_ >> 4;
		  nibble_sel_ = LOW;
		  break;
		default:
		  assert(0);
		  nibble = 0;
	    }

	    lines = 0;
	    if (nibble&1) lines |= PIN_nFault;
	    if (nibble&2) lines |= PIN_Select;
	    if (nibble&4) lines |= PIN_PError;
	    if (nibble&8) lines |= PIN_Busy2;
	    port_regs_->lines = lines;      // Event 8: Write nibble
	    port_regs_->lines = lines|PIN_nAck2;  // Event 9: Lower nAck
	    break;

	      // Event 10: Host raises nAutoFd to accept nibble
	  case PIN_nSelectIn|PIN_nAck2|PIN_nAutoFd:

	    if (id_+nid_ <= curid_)
	      port_regs_->lines |= PIN_nFault;
	    else
	      port_regs_->lines &= ~PIN_nFault;

	    port_regs_->lines &= ~PIN_nAck2; // Event 11: Raise nAck
	    break;

	      // Event X: Host terminates 1284 mode.
	  case PIN_nAutoFd:
	    terminate_ieee1284_();
	    break;

	  default:
	    assert(0);
      }
}
