
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ParallelPort_nibble.cc,v 1.2 1996/10/28 03:45:54 steve Exp $"
/*
 * $Log: ParallelPort_nibble.cc,v $
 * Revision 1.2  1996/10/28 03:45:54  steve
 *  License notice.
 *
 * Revision 1.1  1995/11/02 06:57:21  steve
 *  Split up the ParallelPort source,
 *  Support nibble mode id strings.
 *
 */

# include  <mc68322/ParallelPort.h>
# include  "ParallelPort_priv.h"
# include  <ucr/ucr.h>
# include  <assert.h>


void ParallelPort::enter_nibble_mode_()
{
      port_intr_->enable_flags = LOW_nSelectIn;
      port_isr_ = &port_isr_nibble_;
      reverse_transfer_ = &reverse_nibble_;

        // Wakeup a thread that may be writing to the buffer.
      out_sync_.wakeup();
}

void ParallelPort::port_isr_nibble_()
{
        // If the host terminates the transfer, complete the handshake
	// to go back to compatability mode.
      if (0 == (port_regs_->lines & PIN_nSelectIn))
	terminate_ieee1284_();
}

/*
 * NOTE: This method may be interrupts by nSelectIn going low. It must
 * cope somehow.
 */
void ParallelPort::reverse_nibble_()
{
      const unsigned char byte = *out_buffer_;

        // Event 7: The host lowered nAutoFd to say it is not
	// busy. Send it a byte, a nibble at a time.
      while (port_regs_->lines & PIN_nAutoFd)
	/* busy wait */
	if (! (port_regs_->lines & PIN_nSelectIn)) return;

      unsigned short pins = 0;
      if (byte & 0x01) pins |= PIN_nFault;
      if (byte & 0x02) pins |= PIN_Select;
      if (byte & 0x04) pins |= PIN_PError;
      if (byte & 0x08) pins |= PIN_Busy2;

        // Event 8: Set the low data bits on pins. (nAck is high.)
      port_regs_->lines = pins;

        // Event 9: Set nAck LOW to tell host the pins are ready.
      port_regs_->lines |= PIN_nAck2;

      while (0 == (port_regs_->lines & PIN_nAutoFd))
	/* Wait for event 10: host to ACK bits 0-3 */;

        // Event 11: Raise nAck, first nibble transmitted.
      port_regs_->lines = pins;

      pins = 0;
      if (byte & 0x10) pins |= PIN_nFault;
      if (byte & 0x20) pins |= PIN_Select;
      if (byte & 0x40) pins |= PIN_PError;
      if (byte & 0x80) pins |= PIN_Busy2;

      while (port_regs_->lines & PIN_nAutoFd)
	/* Wait for host ready for bits 4-7 */;

        // Event 8: Set the high data bits on pins
      port_regs_->lines = pins;

        // Event 9: Set nAck LOW to tell host the pins are ready.
      port_regs_->lines |= PIN_nAck2;

      while (0 == (port_regs_->lines & PIN_nAutoFd))
	/* Wait for event 10: host to ACK bits 4-7 */;

      out_remaining_ -= 1;
      out_buffer_ += 1;

        // Event 13: Forward not busy, data available (or not),
        // nAck still low.
      if (out_remaining_ > 0)
	port_regs_->lines = PIN_nAck2;
      else
	port_regs_->lines = PIN_nFault|PIN_PError|PIN_nAck2;

      nap_for_host_();

        // Event 11: Second nibble transmitted. I also
        // transmitted status from event 13.
      port_regs_->lines &= ~PIN_nAck2;
}

