/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ParallelPort_read.cc,v 1.2 1996/10/28 03:45:55 steve Exp $"
/*
 * $Log: ParallelPort_read.cc,v $
 * Revision 1.2  1996/10/28 03:45:55  steve
 *  License notice.
 *
 * Revision 1.1  1995/11/02 06:57:23  steve
 *  Split up the ParallelPort source,
 *  Support nibble mode id strings.
 *
 */


# include  <mc68322/ParallelPort.h>
# include  "ParallelPort_priv.h"
# include  <ucr/ucr.h>
# include  <assert.h>


/*
 * Do a programmed read from the parallel port of the number of
 * bytes. I will enable interrupts here, and the isr will disable them
 * when the buffer is full.
 */
unsigned long ParallelPort::read(void*buf, size_t count)
{
      unsigned char*buffer = (unsigned char*)buf;
      size_t need = count;

      const unsigned short cookie = thread_splhi();

        // If there is a pending transfer (from a readahead) then let
	// it finish. If it is too large, make sure it knows I'm
	// here. Also, by finishing any DMA transfer here, I know that
	// I can start another DMA transfer whenever I want.

      if (transfer_ > need) pdma_regs_->flush_request = 1;
      in_sync_.sleep(&dma_ready_);

      while (need) {
	      // If there are enough bytes ready and waiting, then
	      // simply copy them out.  Easy.
	    if (holding_ >= need) {
		  copyout(buffer, need);
		  need = 0;

	      // Otherwise, copy what I can and arrange to have my job
	      // finished by the DMA controller.
	    } else {

		    // Copyout what is ready
		  size_t ec = holding_;
		  copyout(buffer, ec);
		  need  -= ec;
		  buffer += ec;

		  in_sync_.sleep(&in_flag_);
		  dma_go( (need>nbuf_)? nbuf_ : need );
		  in_sync_.sleep(&dma_ready_);
	    }
      }

      if (in_flag_ && holding_ == 0) dma_go(nbuf_);

      thread_spl(cookie);
      return count;
}

