
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: RGP.cc,v 1.6 1996/10/28 03:45:57 steve Exp $"
/*
 * $Log: RGP.cc,v $
 * Revision 1.6  1996/10/28 03:45:57  steve
 *  License notice.
 *
 * Revision 1.5  1995/10/15 22:53:03  steve
 *  Make better use of builtin sleeps.
 *
 * Revision 1.4  1995/10/13 18:44:02  steve
 *  RGP takes pointers to const.
 *
 * Revision 1.3  1995/09/11 01:07:17  steve
 *  Ask for a bit address in blt2ub_XD
 *  General method for writing longs into program.
 *
 * Revision 1.2  1995/09/08 05:59:10  steve
 *  Encapsulate RGP programs
 *
 * Revision 1.1  1995/09/07 21:48:35  steve
 *  Add the RGP class.
 *
 */


# include  <mc68322/RGP.h>
# include  <ucr/ucr.h>
# include  <assert.h>

const unsigned short RGP_DONE = 0x0002;
const unsigned short RGP_ERROR= 0x0001;
const unsigned short RGP_BUSY = 0x0008;

RGP::RGP(unsigned char level, unsigned long base)
: regs_( (REGS_*)(base + 0x800) ), intr_( (INTR_*)(base + 0x710) )
{
      set_isr(level, isr, this);
      intr_->level = level;
      intr_->intr_flags = 0xffff;
      intr_->enable_flags = RGP_DONE|RGP_ERROR;
}

RGP::~RGP()
{
      intr_->enable_flags = 0;
}

void RGP::go(RGProgram&prog)
{
      const unsigned short cookie = thread_splhi();

      sync_.sleep(intr_->intr_flags, 0, RGP_BUSY);

        // Assign a pointer to the program to the start register to
	// make things happen.
      regs_->start_ = prog.program_;

      sync_.sleep(intr_->intr_flags, 0, RGP_BUSY);

      thread_spl(cookie);
}

void RGP::isr(RGP*obj, unsigned)
{
      unsigned short flags = obj->intr_->intr_flags;
      obj->intr_->intr_flags = flags;

      assert( ! (flags&RGP_ERROR) );

      if (flags) obj->sync_.wakeup();
}


RGProgram::RGProgram(unsigned l, HEAP_SPACE*h)
: heap_(h), program_(new(h)unsigned short[l]), limit_(l), fill_(0)
{
}

RGProgram::~RGProgram()
{
      delete[]program_;
}

void RGProgram::blt2f_D(void*dest, unsigned short width,
			unsigned short height)
{
      st_word(0x1000);
      st_word(((unsigned long)dest>>13) & 0xffff);
      st_word(((unsigned long)dest<<3)  & 0xffff);
      st_word(width);
      st_word(height);
}

void RGProgram::blt2ub_XD(unsigned long dest,
			  unsigned short dwidth, unsigned short dheight,
			  const void*source, unsigned short swarp,
			  unsigned short zoom)
{
      st_word(0x2600);
      st_long(dest);
      st_word(dwidth);
      st_word(dheight);
      st_long((unsigned long)source << 3);
      st_word(swarp);
      st_word(zoom);
}
