/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: fputc.cc,v 1.3 1996/10/28 03:46:03 steve Exp $"
/*
 * $Log: fputc.cc,v $
 * Revision 1.3  1996/10/28 03:46:03  steve
 *  License notice.
 *
 * Revision 1.2  1996/09/16 17:03:07  steve
 *  Add the read/write state machine.
 *
 * Revision 1.1  1996/09/14 00:10:19  steve
 *  Add a stdio library.
 *
 */

# include  <stdio.h>
# include  <StdioDevice.h>
# include  "stdiox.h"
# include  <assert.h>

extern "C" int fputc(int c, FILE*stream)
{
      assert( stream->flags & _FILE::F_WRITE );
      assert( stream->state != _FILE::S_READ );

      if (stream->state == _FILE::S_ERROR) return EOF;
      stream->state = _FILE::S_WRITE;

      unsigned char buf[1];

      switch (stream->buf_mode) {

	      // If the stream is unbuffered, write directly to the
	      // device.
	  case _IONBF:
	    buf[0] = c;
	    stream->dev->write(buf, 1);
	    break;

	      // If the stream is fully buffered, write to the buffer
	      // and only pass data to the device if this fills the
	      // buffer.
	  case _IOFBF:
	    stream->write_to_buf(c);
	    break;

	      // The stream is line buffered, so write the character
	      // to the buffer, and if it is a newline, or the buffer
	      // fills, write the buffer to the device.
	  case _IOLBF:
	    stream->write_to_buf(c);
	    if (c == '\n') fflush(stream);
	    break;
      }

      return c;
}
