
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ISync.cc,v 1.10 1998/01/01 20:45:16 steve Exp $"
/*
 * $Log: ISync.cc,v $
 * Revision 1.10  1998/01/01 20:45:16  steve
 *  ISync has const methods.
 *
 * Revision 1.9  1996/10/28 03:46:08  steve
 *  License notice.
 *
 * Revision 1.8  1996/08/24 20:57:31  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.7  1996/08/10 01:02:17  steve
 *  Test for incorrect call to sleep.
 *
 * Revision 1.6  1996/08/08 05:22:40  steve
 *  Get rid of the __Scheduler class, it was not
 *  needed. Add the thread_runnable and _reschedule
 *  functions for thread scheduling.
 *
 * Revision 1.5  1996/08/03 03:30:37  steve
 *  Create the ucr/asm.h header as a target specific
 *  header with line functions, etc; Replace the
 *  __isr_run_flag with a bit of inline assembly code.
 *
 *  Move much thread code out of the __Secheduler class
 *  and into more public thread_ functions, to eliminate
 *  some wasted function call depth. Even the main_thread
 *  is move into its own file.
 *
 *  Keep a list of available threads for debug use, and
 *  add a function to get thread registers.
 */

# include  <ucr/ISync.h>
# include  <ucr/ucr.h>
# include  <assert.h>

ISync::ISync()
{ }


void ISync::sleep(bool (*fun)(const volatile void*), const volatile void*arg) const
{
      assert( ! isr_hot_flag() );
      spl_cookie_t cookie = thread_splhi();

        // As long as the condition is false, put myself in the stuck_
	// queue (and out of the run queue) and reschedule.
      while(! fun(arg)) {
	    THREAD*thr = thread_self();
	    stuck_.enqueue( thr );
	    thread_reschedule();
      }

      thread_spl(cookie);
}

/*
 * ERROR! This should push the awakened items to the front of the run
 * queue, as they are likely to be real time tasks.
 */
void ISync::wakeup()
{
      spl_cookie_t cookie = thread_splhi();
      THREAD*thr;
      while ( (thr = stuck_.pull()) ) {
	    thread_runnable(thr);
      }

      thread_reschedule();
      thread_spl(cookie);
}
