
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ThreadQueue.cc,v 1.6 1997/08/11 22:39:15 steve Exp $"
/*
 * $Log: ThreadQueue.cc,v $
 * Revision 1.6  1997/08/11 22:39:15  steve
 *  Threads that finish are dequeued.
 *
 * Revision 1.5  1996/10/28 03:46:11  steve
 *  License notice.
 *
 * Revision 1.4  1996/09/11 20:20:15  steve
 *  Minor optimizations
 *
 * Revision 1.3  1996/08/08 20:57:23  steve
 *  Minor optimizations by inlining a few things
 *  and setting optimization attributes.
 *
 * Revision 1.2  1996/08/08 05:22:41  steve
 *  Get rid of the __Scheduler class, it was not
 *  needed. Add the thread_runnable and _reschedule
 *  functions for thread scheduling.
 *
 * Revision 1.1  1995/09/02 18:27:23  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

# include  <ucr/ThreadQueue.h>
# include  "__thread.h"

/*
 * The threadqueue is a double circularly linked list. The last_
 * pointer points to the last cell in the queue, or is 0 if there are
 * no things in the queue. The first thing in the queue is the cell
 * after the last, otherwise known as last_->next_.
 *
 * Also, a cell can only be in one queue at a time. If it is in
 * another when enqueue is called, it will be first pulled from the
 * other list. If it is already in the current list, it will be moved
 * to the back (or front if the flag is set).
 */

void ThreadQueue::pull_(THREAD*thr)
{
      if (last_ == thr) last_ = thr->prev_;

      if (last_ == thr) {
	    last_ = 0;
	    return;
      }

      thr->next_->prev_ = thr->prev_;
      thr->prev_->next_ = thr->next_;

}

ThreadQueue::ThreadQueue(THREAD*thr)
{
      if (thr->owner_) thr->owner_->pull_(thr);
      thr->owner_ = this;
      thr->next_ = thr;
      thr->prev_ = thr;
      last_ = thr;
}

void ThreadQueue::enqueue(THREAD*thr, bool front)
{
      if (thr->owner_) thr->owner_->pull_(thr);

      thr->owner_ = this;

      if (last_ == 0) {
	    last_ = thr;
	    thr->next_ = thr;
	    thr->prev_ = thr;
	    return;
      }

      thr->next_ = last_->next_;
      thr->prev_ = last_;
      thr->next_->prev_ = thr;
      thr->prev_->next_ = thr;

        // The insertion above put the cell after the cell that last_
	// points to. That would make it the front. If I want to put
	// it in back, then move the last_ pointer.
      if (front) return;
      last_ = thr;
}

THREAD* ThreadQueue::pull()
{
      if (last_ == 0) return 0;
      THREAD*thr = last_->next_;
      pull_(thr);
      thr->owner_ = 0;
      return thr;
}

THREAD* ThreadQueue::peek()
{
      if (last_ == 0) return 0;
      return last_->next_;
}

void ThreadQueue::dequeue(THREAD*thr)
{
      if (thr->owner_) thr->owner_->pull_(thr);
}
