
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: malloc.cc,v 1.7 1996/10/28 03:46:28 steve Exp $"
/*
 * $Log: malloc.cc,v $
 * Revision 1.7  1996/10/28 03:46:28  steve
 *  License notice.
 *
 * Revision 1.6  1996/08/24 20:57:32  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.5  1996/05/24 01:46:48  steve
 *  Initial port to i960_cyclone
 *  Seperate some files in the library to seperate files
 *  for making smaller binaries, use uCR_ prefix for private
 *  symbols, move assembly files around for compile-time
 *  portability, change __start to ucr_start and give it
 *  new behavior, and simplify the ucr_assert implementation.
 *
 * Revision 1.4  1995/10/20 03:59:37  steve
 *  Safe heap stats.
 *
 * Revision 1.3  1995/10/17 21:27:01  steve
 *  Use stddef instead of stdlib,
 *  get stats for default heap.
 *
 * Revision 1.2  1995/09/08 05:58:22  steve
 *  Better integration of heap handling with
 *  the new/delete operators.
 */

# include  <ucr/heap.h>
# include  <ucr/ucr.h>


extern HEAP_SPACE*uCR_malloc_heap;

/*
 * Malloc and free, however, need to be thread safe, and the way to do
 * that is to protect them from interrupts.
 */

extern "C" void* malloc(size_t size)
{
      spl_cookie_t cookie = thread_splhi();
      void*result = heap_alloc(size, uCR_malloc_heap);
      thread_spl(cookie);
      return result;
}

extern "C" void free(void*memory)
{
      spl_cookie_t cookie = thread_splhi();
      heap_free(memory);
      thread_spl(cookie);
}
