/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: qsort.cc,v 1.1 1997/04/30 16:53:21 steve Exp $"
/*
 * $Log $
 */

# include  <stdlib.h>

static void*findpivot(void*left, void*right, size_t size,
		      int (*compar)(const void*, const void*))
{
      void*pivot = left+size;
      while (pivot <= right) {
	    int rc = compar(left, pivot);
	    if (rc == 0)
		  pivot += size;
	    else if (rc < 0)
		  return pivot;
	    else
		  return left;
      }

      return 0;
}

static void swap(void*left, void*right, size_t size)
{
      char tmp;
      char*l = (char*)left;
      char*r = (char*)right;

      for ( ; size > 0 ;  size -= 1, l+= 1, r += 1) {
	    tmp = *l;
	    *l = *r;
	    *r = tmp;
      }
}

extern "C" void qsort(void*base, size_t nmemb, size_t size,
		      int (*compar)(const void*, const void*))
{
      if (nmemb < 2) return;
      void*left = base;
      void*right = base + size*(nmemb-1);

      void*pivot = findpivot(left, right, size, compar);
      if (pivot == 0) return;

      for (;;) {
	    while ( compar(left, pivot) < 0 ) left += size;
	    while ( compar(pivot, right) < 0) right -= size;

	    if (left > right) break;

	    swap(left, right, size);
      }

      qsort(base, (left-base)/size-1, size, compar);
      qsort(left, nmemb - (left-base)/size, size, compar);
}
