/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: set_fault.cc,v 1.7 1998/05/26 16:03:36 steve Exp $"
/*
 * $Log: set_fault.cc,v $
 * Revision 1.7  1998/05/26 16:03:36  steve
 *  Add panic.
 *
 * Revision 1.6  1998/04/03 01:35:12  steve
 *  Add assertion support.
 *
 * Revision 1.5  1996/11/07 02:29:20  steve
 *  Use more portable methods.
 *
 * Revision 1.4  1996/10/28 03:46:34  steve
 *  License notice.
 *
 * Revision 1.3  1996/08/24 20:57:33  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.2  1996/08/14 00:24:50  steve
 *  Add fault information to the THREAD object so that
 *  it can be retrieved at any time.
 *
 * Revision 1.1  1996/08/10 22:06:56  steve
 *  Add generic fault handling code.
 *
 */

# include  <ucr/fault.h>
# include  <ucr/ucr.h>
# include  "__thread.h"
# include  <assert.h>
# include  <stdlib.h>

static struct {
      ucr_fault_handler fun;
      void*arg;
} handlers[FAULT_COUNT];

void set_fault(unsigned num, ucr_fault_handler fun, void*arg)
{
      assert(num < FAULT_COUNT);
      handlers[num].fun = fun;
      handlers[num].arg = arg;
}

extern "C" void uCR_fault_base(FAULT_RECORD*fault)
{
      THREAD*self = thread_self();
      if (isr_hot_flag())
	    ucr_panic("fault in ISR", fault->ip);
      assert( self->fault_ == 0 );
      assert( self->bstack < fault );
      assert( fault < self->estack );
      self->fault_ = fault;

      bool resume = false;
      unsigned type = fault_type(fault);
      if (handlers[type].fun)
	    resume = handlers[type].fun(handlers[type].arg);

      if (resume) { self->fault_ = 0; return; }
      else assert(0);
}
