#ifndef __thread_cookie_H
#define __thread_cookie_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: thread_cookie-sparc.h,v 1.1 1996/11/08 21:40:44 steve Exp $"
/*
 * $Log: thread_cookie-sparc.h,v $
 * Revision 1.1  1996/11/08 21:40:44  steve
 *  Divide up thread_create for easier porting, Add the
 *  sparc_linux and i386_linux targets, and adjust makefiles
 *  to allow for them. Document the new concept of virtual boards.
 *
 *  Change the linker output file extension to .exe, because
 *  some targets use ELF and not COFF.
 *
 *  Get rid of some trivial warnings.
 *
 */

/*
 * The sparc thread context is the global registers (%r0-%r7) and the
 * out regs (%r8-%r15). A thread is restored like so:
 *
 * flush the windows (Stoftware trap 3)
 * load %g0-%g7 from the cookie
 * load %o0-%o7 from the cookie into %i0-%i7
 * restore (This causes the %i0-%i7 registers to be moved to %o0-%o7,
 *          their proper place, and loads %i0-%i7 and %l0-%l7 from the
 *          stack frame.)
 * ret (jmp to %i7 + 8)
 *
 * Loading the %o0-%o7 register into %i0-%i7 sets up the suspended
 * stack pointer (%sp) as the frame pointer (%fp) and the restore
 * shifts me into the desired frame.
 *
 * NOTE: Since interrupts are not yet supported, the CC flags are not
 * important (the compiler knows they will be trashed) and I do not
 * save the PSW. When I do a real sparc implementation, I will need to
 * add PSR switching.
 *
 * NOTE on GLOBAL REGISTERS: In spite of recommendations in the SPARC
 * Architecture Manual (Version 8) on the use of global registers, the
 * GNU compiler may choose to allocate temporaries in the global
 * registers. The architecture manual is not crystal clear on what
 * SPARC International wants those registers reserved for, but its
 * likely not something a uCR programmer wants. Therefore, uCR will
 * treat global registers as temporaries (gcc model) and preserve them
 * in thread switches.
 */
# define REGISTER_COUNT 16

/*
 * The registers %g0-%g7 and %o0-%o7 are stored here, in that
 * order. The SPARC_REG_x macros return an index into the array for the
 * register of interest. SPARC_REG_GLOBAL(0) need not save %g0 so is
 * used for the %y register instead.
 *
 * The %l0-%l7 and %i0-%i7 registers are saved on the stack frame.
 */

# define SPARC_REG_OUT(x) ((x) + 8)
# define SPARC_REG_GLOBAL(x) ((x) + 0)

struct ThreadCookie {
      unsigned long regs [REGISTER_COUNT];
};

# define COOKIE_INITIALIZER { { 0 } }

/*
 * The uCR_stack_init touches everything there that is of interest, so
 * this has nothing to do. How nice.
 */
static inline void thread_cookie_init(ThreadCookie* /*cookie*/)
{
}



#endif
