
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: thread_create.cc,v 1.17 1997/08/11 22:39:15 steve Exp $"
/*
 * $Log: thread_create.cc,v $
 * Revision 1.17  1997/08/11 22:39:15  steve
 *  Threads that finish are dequeued.
 *
 * Revision 1.16  1996/11/11 03:48:05  steve
 *  Optimize for size.
 *
 * Revision 1.15  1996/11/08 21:40:44  steve
 *  Divide up thread_create for easier porting, Add the
 *  sparc_linux and i386_linux targets, and adjust makefiles
 *  to allow for them. Document the new concept of virtual boards.
 *
 *  Change the linker output file extension to .exe, because
 *  some targets use ELF and not COFF.
 *
 *  Get rid of some trivial warnings.
 *
 * Revision 1.14  1996/10/28 03:46:40  steve
 *  License notice.
 *
 * Revision 1.13  1996/09/19 04:44:16  steve
 *  Add the i960_ise target, change the target
 *  configuration headers to allow sharing by
 *  targets, fix up the make clean rule.
 *
 * Revision 1.12  1996/09/17 21:51:40  steve
 *  Add support for errno.h
 *
 * Revision 1.11  1996/08/25 00:29:11  steve
 *  Optimize thread switching.
 *
 * Revision 1.10  1996/08/24 20:57:35  steve
 *  The priority cookie is not necessarily short, so make
 *  a typedef for it.
 *
 *  There is no need (indeed no way) to save and restore the
 *  fp of a thread, so put the PFP, which is really the fp,
 *  into the fp register slot.
 *
 *  Slightly better check of the stack frame at thread switch
 *  time. Also have faults check for some bad things.
 *
 * Revision 1.9  1996/08/22 00:36:11  steve
 *  the stack pointer is not handled seperately.
 *
 * Revision 1.8  1996/08/14 00:24:51  steve
 *  Add fault information to the THREAD object so that
 *  it can be retrieved at any time.
 *
 * Revision 1.7  1996/08/08 20:57:24  steve
 *  Minor optimizations by inlining a few things
 *  and setting optimization attributes.
 *
 * Revision 1.6  1996/08/03 03:30:44  steve
 *  Create the ucr/asm.h header as a target specific
 *  header with line functions, etc; Replace the
 *  __isr_run_flag with a bit of inline assembly code.
 *
 *  Move much thread code out of the __Secheduler class
 *  and into more public thread_ functions, to eliminate
 *  some wasted function call depth. Even the main_thread
 *  is move into its own file.
 *
 *  Keep a list of available threads for debug use, and
 *  add a function to get thread registers.
 *
 * Revision 1.5  1996/07/30 03:17:47  steve
 *  Add the string.h header, and strlen.
 *
 * Revision 1.4  1996/05/29 22:52:53  steve
 *  Simplify and optimize i960 thread switching.
 *
 * Revision 1.3  1996/05/27 22:50:02  steve
 *  implement thread_create and thread_switch for the i960,
 *  rename some __foo symbols to uCR_foo for neatness.
 */

# include  <ucr/ucr.h>
# include  <ucr/ThreadQueue.h>
# include  <assert.h>
# include  <string.h>
# include  "__thread.h"

extern void uCR_stack_init(THREAD*tid,
			   void (*base)(void (*)(void*), void*),
			   void (*fun)(void*), void*arg);

static void base_of_thread(void (*fun)(void*), void*arg);


/*
 * Initialize the thread structure, which I steal from the top of the
 * stack. Build up an initial stack frame and place it there. When
 * done, this thread is ready to be executed. The initial stack frame
 * is constructed so that a return from uCR_switch resumes the thread
 * at the first instruction of base_of_thread with the correct
 * parameters ready to go.
 */
THREAD * thread_create(void*stack, unsigned size, void (*fun)(void*),
		       void*arg)
{
      assert( stack );
      assert( size > sizeof(THREAD) );

        // Pull the thread structure off the top of the stack. The
	// stack pointer will start below this and work down for
	// stacks that grow down, or bstack and work up for stacks
	// that grow up.
      THREAD*tid = (THREAD*)((char*)stack + size - sizeof(THREAD));
      tid->bstack = stack;
      tid->estack = tid;
      tid->errno_ = 0;
      tid->fault_ = 0;
      tid->owner_ = 0;
      tid->next_  = 0;
      tid->prev_  = 0;
	//tid->list_  = 0; // Deal with this later, when the thread is done.

      thread_cookie_init(&tid->cookie);

      uCR_stack_init(tid, &base_of_thread, fun, arg);

	// Now that the thread is all made, put it in the thread list.
      spl_cookie_t cookie = thread_splhi();
      THREAD*self = thread_self();
      tid->list_ = self->list_;
      self->list_ = tid;
      thread_spl(cookie);

      return tid;
}

static void base_of_thread(void (*fun)(void*), void*arg)
{
      fun(arg);

	// If the thread completes, dequeue and run the scheduler to
	// let something else go. This thread should never again go.
      ThreadQueue::dequeue(thread_self());
      thread_reschedule();
      abort();
}
