
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: uCR_stack-i386.cc,v 1.3 1996/11/11 03:48:06 steve Exp $"
/*
 * $Log: uCR_stack-i386.cc,v $
 * Revision 1.3  1996/11/11 03:48:06  steve
 *  Optimize for size.
 *
 * Revision 1.2  1996/11/10 23:21:09  steve
 *  Fix stack frame of fresh thread.
 *
 * Revision 1.1  1996/11/09 04:34:26  steve
 *  Add i386_linux thread infrastructure.
 *
 */

# include  "__thread.h"
# include  <string.h>

/*
 * I want to create the situation where, after a thread switch, all
 * the registers look like this:
 *
 *	%eax  = 0
 *	%edx  = 0
 *	%ecx  = 0
 *	%ebx  = 0
 *	%ebp  = 0
 *	%esi  = 0
 *	%edi  = 0
 *	%esp  = tid->estack - 12 bytes (Point to the arguments)
 *
 *	 EIP  = base
 *
 *	tid->estack[-4bytes]  = arg (second argument)
 *	tid->estack[-8bytes]  = fun (first argument)
 *	tid->estack[-12bytes] = 0   (no place to return to)
 *
 * This is what the stack frame would look like if the base was called
 * by a non-existent function, with fun and arg as parameters, and the
 * first instrruction of base has not been executed yet.
 *
 * Furthermore, the thread switch enters the thread by loading the
 * registers from the save area and executing:
 *
 *	ret
 *
 * to pop the frame and jump into the thread code. So, start with the
 * desired state of affairs and run the program backwards. The ``ret''
 * loads the EIP from the top of the stack, so push base into the top
 * of the stack and adjust %esp, so finally we get:
 *
 *	%eax  -- 0
 *	%edx  -- 0
 *	%ecx  -- 0
 *	%ebx  -- 0
 *	%ebp  -- 0
 *	%esi  -- 0
 *	%edi  -- 0
 *	%esp  -- tid->estack - 16 bytes (``ret'' will increment this.)
 *
 *	tid->estack[-4bytes]  = arg
 *	tid->estack[-8bytes]  = fun
 *	tid->estack[-12bytes] = 0
 *      tid->estack[-16bytes]  -- base (``ret'' pops this into EIP)
 */
void uCR_stack_init(THREAD*tid,
		    void (*base)(void (*)(void*), void*),
		    void (*fun)(void*), void*arg)
{
      unsigned long*const esp = (unsigned long*) tid->estack;

      memset(tid->cookie.regs, 0, sizeof tid->cookie.regs);
      tid->cookie.regs[I386_REG_ESP] = (unsigned long)(esp - 4);

      esp[-1] = (unsigned long)arg;
      esp[-2] = (unsigned long)fun;
      esp[-3] = 0;
      esp[-4] = (unsigned long)base;
}
