
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ucr_start.c,v 1.9 1998/06/30 00:03:45 steve Exp $"
/*
 * $Log: ucr_start.c,v $
 * Revision 1.9  1998/06/30 00:03:45  steve
 *  Fix definition of ctor tables.
 *
 * Revision 1.8  1997/09/04 07:37:54  steve
 *  Get rid of excessive _ in xTOR_TABLE symbols.
 *
 * Revision 1.7  1996/10/28 03:46:48  steve
 *  License notice.
 *
 * Revision 1.6  1996/09/19 04:44:23  steve
 *  Add the i960_ise target, change the target
 *  configuration headers to allow sharing by
 *  targets, fix up the make clean rule.
 *
 * Revision 1.5  1996/08/10 04:06:27  steve
 *  Inline the thread_spl things where possible,
 *  stop basing heap on estack, and fix startup
 *  code to use end_heap.
 *
 * Revision 1.4  1996/07/30 03:17:48  steve
 *  Add the string.h header, and strlen.
 *
 * Revision 1.3  1996/05/29 04:29:04  steve
 *  Portable way to ENABLE interrupts.
 *
 * Revision 1.2  1996/05/27 22:38:08  steve
 *  Always initialize the .bss section.
 *
 * Revision 1.1  1996/05/24 01:46:53  steve
 *  Initial port to i960_cyclone
 *  Seperate some files in the library to seperate files
 *  for making smaller binaries, use uCR_ prefix for private
 *  symbols, move assembly files around for compile-time
 *  portability, change __start to ucr_start and give it
 *  new behavior, and simplify the ucr_assert implementation.
 *
 * Revision 1.1  1995/09/02 18:27:27  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

# include  <string.h>
# include  <ucr/config.h>

extern const char etext;
extern char bdata, edata, end, end_heap;

extern void (*__CTOR_TABLE__[])() asm("__CTOR_TABLE__");
extern void (*__DTOR_TABLE__[])() asm("__DTOR_TABLE__");

extern int main();

extern void uCR_malloc_init(void*heap, unsigned long heap_size);


/*
 * copy the boot initialized data into the data section of RAM. This
 * allows the data section to come from ROM.
 */

int ucr_start()
{
      int main_rc;
      int idx;

	/* There are certain cases where I do not need to copy the
	   data image. The user tells me this by putting bdata up
	   against etext. At any rate, initialize the .bss section by
	   writing 0s into it. */

      if (&bdata != &etext) memcpy(&bdata, &etext, &edata - &bdata);
      memset(&edata, 0, &end - &edata);

      uCR_malloc_init(&end, &end_heap-&end);
      
      for (idx = 0 ;  __CTOR_TABLE__[idx] ;  idx += 1)
	(__CTOR_TABLE__[idx])();

        /* Enable interrupts. */
      thread_spl_low();

      main_rc = main();

	/* Disable interrupts. */
      thread_splhi();

      for (idx = 0 ;  __DTOR_TABLE__[idx] ;  idx += 1)
	(__DTOR_TABLE__[idx])();

      return main_rc;
}
