/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: udlopen.cc,v 1.2 1997/09/04 01:05:45 steve Exp $"

# include  <chips/BUS.h>
# include  "__udlfcn.h"
# include  <stdlib.h>
# include  <assert.h>

struct Head {
      unsigned long cmd;
      unsigned long parm1;
      unsigned long parm2;
      unsigned long len;
};


struct udl_handle* udlopen(BUS::Channel&chan)
{
      Head head;
      udl_handle*mem = 0;
      size_t mod_size;

      for (;;) {
	    chan.read(&head, sizeof head);

	    switch (head.cmd) {

		case 1:
		    // Write the symbol table to the host. The host
		    // will use this to do a trial link and get an
		    // image size.
		  chan.write(symbol_table, symbol_table[1]);
		  break;

		case 2:
		  if (mem) free(mem);

		    // Read from the host the amount of memory needed
		    // for the module. This is in bytes.

		  mod_size = (head.parm1 + 63) & ~0x1f;

		    // Make the module space, and send the assigned
		    // location and size back to the host for a final
		    // link. The value actually sent back to the host
		    // is aligned to 16byte boundries for the linker's
		    // sake.
		  mem = (udl_handle*)malloc(mod_size + sizeof(udl_handle));
		  mem->mod.size = mod_size;
		  if ((mem->mod.loc = (unsigned long)(mem+1)) & 0x0f) {
			ptrdiff_t offst = 0x10 - (mem->mod.loc & 0x0f);
			mem->mod.loc += offst;
			mem->mod.size -= offst;
		  }

		  chan.write(&mem->mod, sizeof mem->mod);
		  break;

		case 3:

		    // Read the module into the space previously set
		    // aside for it. The host sends the download
		    // command along with the load address, to make
		    // sure we got it right. The host also sends the
		    // address of the symbol table, so the udlsym can
		    // find symbol values.

		  assert( mem );
		  assert( head.parm1 == mem->mod.loc );
		  assert( head.len <= mem->mod.size );

		  chan.read((void*)mem->mod.loc, head.len);

		  mem->symbol_table = (unsigned long*)head.parm2;

		    // Invoke the init function for the module.
		  ((funp*)mem->mod.loc)[0]();
		  return mem;

		default:
		  assert(0);
	    }
      }

      return mem;
}

/*
 * $Log: udlopen.cc,v $
 * Revision 1.2  1997/09/04 01:05:45  steve
 *  Add the udlsym function.
 *
 * Revision 1.1  1997/09/03 23:45:03  steve
 *  Add support for dynamic loading, including the
 *  commands to make symbol tables, read and link
 *  modules, and download fondled binaries.
 *
 */

