#!tclpci -f

# --
# Copyright (c) 1996 Picture Elements, Inc.
# Stephen Williams (steve@icarus.com)
#
#    This source code is free software; you can redistribute it
#    and/or modify it in source code form under the terms of the GNU
#    General Public License as published by the Free Software
#    Foundation; either version 2 of the License, or (at your option)
#    any later version. In order to redistribute the software in
#    binary form, you will need a Picture Elements Binary Software
#    License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
#    You should also have recieved a copy of the Picture Elements
#    Binary Software License offer along with the source. This offer
#    allows you to obtain the right to redistribute the software in
#    binary (compiled) form. If you have not received it, contact
#    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
#
# $Id: reset_ise.tcl,v 1.2 1996/10/28 03:46:51 steve Exp $
#
# $Log: reset_ise.tcl,v $
# Revision 1.2  1996/10/28 03:46:51  steve
#  License notice.
#
# Revision 1.1  1996/10/10 04:28:17  steve
#  Add the ability the reset the ISE board from software,
#  using the pciconf package.
#

# --
# This program uses the pciconf driver to reset the ISE board. This
# includes starting it initially from its power-up hold state or
# restarting it to leave an application and get back into the boot
# prom.
#
# It works by manipulating the local bus reset bit in the extended
# bridge control register of the bridge part of the i960RP.

pciopen bridge -r -w /dev/pciconf
bridge select 0x8086 0x0960 0

# --
# If the i960RP is not yet being held reset, then I must force it
# reset myself. This will trash the configuration space registers for
# the ATU, so I must remember to restore them.

if {([bridge get_byte 0x40] & 0x02) == 0} {

    pciopen ATU -r -w /dev/pciconf
    ATU select 0x8086 0x1960 0

    set reg_list "0x04 0x0c 0x10 0x30 0x3c"

    foreach idx $reg_list { set save($idx) [ATU get_dword $idx] }

    # This reset the local bus on the i960RP...
    bridge set_byte 0x40 0x20

    # and this restores the registers that the BIOS set up.
    foreach idx $reg_list { ATU set_dword $idx $save($idx) }
}

# --
# All set. Release the CPU.
bridge set_byte 0x40 0
