#ifndef __os_winnt_H
#define __os_winnt_H
/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#if !defined(WINNT)
#ident "$Id: os-nt.h,v 1.13 1998/04/25 05:05:26 steve Exp $"
#endif

#include  <ntddk.h>
#include  "ntmsg.h"

extern void wake_up(struct wait_queue**);

/* MS products to not seem to support inline. */
# define __inline__

/* MS products deal with processor compatibility by defining them to
   all be 32bit ix86. Thus, specific size types are kinda obvious. */
typedef unsigned long  __u32;
typedef unsigned short __u16;

/* A buffer accessible to both the driver and the device may have two
   different addresses, depending on who you are. This structure holds
   both addresses for the same buffer. */
struct shared_buf {
      void*host;
      __u32 bus;
};

struct shared_buf alloc_buf();
void free_buf(struct shared_buf);

/* Fix the max size of a single buffer sent to the target or made
   available to it. */
#define PACKET_MTU PAGE_SIZE

/* Define some compatibility constants. */
# define EBUSY 1
# define EINTR 2
# define EINVAL 3
# define ENOTTY 4
# define EIO 5
# define ENOMEM 6
# define EAGAIN 7

#define readl(off) READ_REGISTER_ULONG((unsigned long*)(off))
#define writel(val,off) WRITE_REGISTER_ULONG((unsigned long*)(off),(val))

/* The various support in the winnt stuff arranges for the memory in
   all cases to be mapped into kernel virtual address space. Thus,
   copying is rather simple. */
#define copy_to_user(d,s,c) RtlCopyMemory((d), (s), (c))
#define copy_from_user(d,s,c) RtlCopyMemory((d),(s),(c))


/* There are cases where I don't want the blocking routines to
   actually block. If the code detects a case where it should block
   but has been asked not to, the would_have_blocked flag is set. */
#define is_signalled(xsp) ((xsp)->would_have_blocked)

/* These are version of the BIOS32 functions for NT */
extern int pcibios_read_config_dword(unsigned char bus, unsigned char dev_fn,
				     unsigned char off, unsigned*val);
extern int pcibios_read_config_word(unsigned char bus, unsigned char dev_fn,
				    unsigned char off, unsigned short*val);
extern int pcibios_read_config_byte(unsigned char bus, unsigned char dev_fn,
				    unsigned char off, unsigned char*val);
extern int pcibios_write_config_dword(unsigned char bus, unsigned char dev_fn,
				      unsigned char off, unsigned val);
extern int pcibios_write_config_byte(unsigned char bus, unsigned char dev_fn,
				     unsigned char off, unsigned char val);
# define PCIBIOS_SUCCESSFUL 0x00
# define PCIBIOS_DEVICE_NOT_FOUND 0x86
# define PCIBIOS_BAD_REGISTER_NUMBER 0x87

/* The win nt version of the driver needs these members stored in the
   instance to remember the irq information for later unload, and also
   needs to remember if MmMap... was used to map the registers. */
#define OS_INST_MEMBERS \
    int dev_is_mmapped; \
    KIRQL irql; \
    PKINTERRUPT irq; \
    BOOLEAN blocking_is_ok; \
    BOOLEAN would_have_blocked; \
    LIST_ENTRY pending_read_irps, pending_write_irps, tmp_queue;\
    KDPC pending_io_dpc;

#define OS_CHAN_MEMBERS \
    unsigned long block_flag; \
    IRP*irp;

/* Take the device_fn of the i960RP ATU and generate the matching
   device_fn for the bridge. This works by subtracting 1 from the
   function unit. */
#define i960rp_to_bridge(x) ((x)-0x20)

/* This is the type code for the device object that represents the
   physical board. Instance structures are connected to these devices. */
#define FILE_DEVICE_ISE  0x8001
extern NTSTATUS create_device(DRIVER_OBJECT*drv, unsigned bus_no,
			      unsigned slot, unsigned num);
extern void unload_device(DEVICE_OBJECT*dev);

extern NTSTATUS dev_create(DEVICE_OBJECT*dev, IRP*irp);
extern NTSTATUS dev_close(DEVICE_OBJECT*dev, IRP*irp);
extern NTSTATUS dev_read(DEVICE_OBJECT*dev, IRP*irp, BOOLEAN retry_flag);
extern NTSTATUS dev_write(DEVICE_OBJECT*dev, IRP*irp, BOOLEAN retry_flag);
extern NTSTATUS dev_ioctl(DEVICE_OBJECT*dev, IRP*irp);

/* The UCRX driver is a control path into the target board. Only the
   ioctl operation is supported here in general. */
#define FILE_DEVICE_UCRX 0x8003
extern NTSTATUS create_devx(DRIVER_OBJECT*drv, struct Instance*xsp);
extern void unload_devx(DEVICE_OBJECT*dev);

extern NTSTATUS devx_ioctl(DEVICE_OBJECT*dev, IRP*irp);


/* The ISE driver has a single console device (independent of the
   number of physical ISE boards) that is used to carry driver debug
   messages. */
#define FILE_DEVICE_ISE_CONS 0x8002
extern NTSTATUS create_console(DRIVER_OBJECT*drv);

/* The console device has these dispatch routines, and support the
   debug output operations. These definitions relate to that
   functionality. */

extern void unload_console(DEVICE_OBJECT*dev);
extern NTSTATUS cons_read(DEVICE_OBJECT*dev, IRP*irp);

extern void printk(const char*fmt, ...);

/* This function initializes the NT frame pool with the given size. The frame
   pool is preallocated because NT is unlikely to give me the space othersize. */
extern int ucr_init_nt_frame_pool(unsigned long size);
extern void ucr_deinit_nt_frame_pool();


/*
 * $Log: os-nt.h,v $
 * Revision 1.13  1998/04/25 05:05:26  steve
 *  Preallocate frame space under NT.
 *
 * Revision 1.12  1998/04/22 19:06:57  steve
 *  Some write cancel support.
 *
 * Revision 1.11  1998/04/18 02:43:54  steve
 *  Fix trouble removing device lines at unload.
 *  Add support for read cancellation under NT.
 *  isecons device (nt only) now throws away old
 *  data instead of truncating new.
 *
 * Revision 1.10  1998/03/23 03:30:24  steve
 *  non-blocking reading under NT.
 *
 * Revision 1.9  1998/03/23 00:07:38  steve
 *  initial frame support for winnt.
 *
 * Revision 1.8  1998/03/21 21:06:43  steve
 *  Rearrange frame handling code.
 *
 * Revision 1.7  1997/12/07 02:16:42  steve
 *  Fix NT blocking problems, redo the console to
 *  not block and keep a limited buffer, trace the
 *  ucrx operations, and ucrx operations work on NT.
 *
 * Revision 1.6  1997/12/06 05:15:51  steve
 *  Add the control device to the NT port.
 *
 * Revision 1.5  1997/12/02 02:29:03  steve
 *  Add support for ISE board reset.
 *
 * Revision 1.4  1997/11/16 11:37:43  steve
 *  Make NT driver asynchronous.
 *
 * Revision 1.3  1997/11/11 11:19:46  steve
 *  Add support for frames.
 *
 * Revision 1.2  1997/09/12 05:18:06  steve
 *  Port to linux 2.1
 *
 * Revision 1.1  1997/09/11 21:58:36  steve
 *  Port the uCR device driver to Windows NT,
 *  make the configure select the target board
 *  and target host interface headers.
 *
 */


#endif
