/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the
 *    Free Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: basic_threads.cc,v 1.4 1997/09/02 21:49:35 steve Exp $"
/*
 * $Log: basic_threads.cc,v $
 * Revision 1.4  1997/09/02 21:49:35  steve
 *  Fix broken thread test.
 *
 * Revision 1.3  1996/11/12 00:38:55  steve
 *  Implement the sparc longjmp, fix the IEEEFP byte
 *  extraction for sparc, add the UCR_STACK_ATTRIBUTE
 *  definition and get the regress test to work for sparc_linux.
 *
 * Revision 1.2  1996/11/11 05:39:09  steve
 *  Get the target-specific main into a seperate function
 *  in order to do a stop on some targets, and make the
 *  thread test more synchronous.
 *
 * Revision 1.1  1996/11/10 23:38:48  steve
 *  Add a regression test to uCR.
 *
 */

# include  <ucr/ucr.h>
# include  <ucr/ISema.h>
# include  <stdio.h>
# include  <assert.h>

static ISema test_sync0 (0);
static ISema test_sync1 (0);

unsigned long test_stack[256] __attribute__ ((UCR_STACK_ATTRIBUTE));

static void test(void*arg)
{
      assert( 0xa5a5 == (unsigned long)arg );

      test_sync0.decr();
      test_sync1.incr();
}

void basic_threads(FILE*out)
{
      fprintf(out, "... Create a thread.\n");
      fflush(out);

      THREAD*tid = thread_create(test_stack, sizeof test_stack, test,
				 (void*)0xa5a5);
      assert(tid);
      thread_runnable(tid);
      test_sync0.incr();

      fprintf(out, "... Excite thread by blocking on semaphore\n");
      fflush(out);
      test_sync1.decr();

      fprintf(out, "... Excite thread by doing a thread_yield()\n");
      fflush(out);
      thread_yield();
}
