/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 */
#ident "$Id: ieee_functions.cc,v 1.1 1996/11/10 23:38:51 steve Exp $"
/*
 * $Log: ieee_functions.cc,v $
 * Revision 1.1  1996/11/10 23:38:51  steve
 *  Add a regression test to uCR.
 *
 */

/*
 * This function tests the IEEE math functions to make sure they
 * generate the correct results in as many different cases as I can
 * think of.
 */

# include  <math.h>
# include  <stdio.h>
# include  <assert.h>

static void floor_ceil_test(double x)
{
      double y;

      y = floor(x + 0.5);
      assert( x == y );

      y = ceil(x - 0.5);
      assert( x == y );

      y = floor(x - 0.5);
      assert( y == (x-1.0) );

      y = ceil(x + 0.5);
      assert( y == (x+1.0) );
}

void ieee_functions(FILE*out)
{
      double x, y;

      fprintf(out, "... isnan() and finite()\n");
      fflush(out);

      assert( ! isnan(0.0) );
      assert( finite(0.0) );

      fprintf(out, "... floor() and ceil()\n");
      fflush(out);

      y = floor(0.0);
      assert( y == 0.0 );

      floor_ceil_test( 0.00);
      floor_ceil_test( 1.00);
      floor_ceil_test(-1.00);
      floor_ceil_test( 5.00);
      floor_ceil_test(-5.00);

      fprintf(out, "... nextafter()\n");
      fflush(out);

      y = nextafter(0.0, 1.0);
      assert( y > 0.0 );

      y = nextafter(0.0, -1.0);
      assert( y < 0.0 );

      y = nextafter(1.0, 2.0);
      assert( 1.0 < y );
      assert( y < 2.0 );

      x = nextafter(2.0, 1.0);
      assert( 1.0 < x );
      assert( x < 2.0 );
      assert( y < x );

      y = floor(nextafter(1.0, 2.0));
      assert( 1.0 == y );

      y = floor(nextafter(2.0, 1.0));
      assert( 1.0 == y );
}
