/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: main-i960_cyclone.cc,v 1.1 1997/02/21 21:21:00 steve Exp $"
/*
 * $Log: main-i960_cyclone.cc,v $
 * Revision 1.1  1997/02/21 21:21:00  steve
 *  Regression test for cyclone
 *
 * Revision 1.2  1996/11/11 05:39:11  steve
 *  Get the target-specific main into a seperate function
 *  in order to do a stop on some targets, and make the
 *  thread test more synchronous.
 *
 * Revision 1.1  1996/11/10 23:38:52  steve
 *  Add a regression test to uCR.
 *
 */

/*
 * The Cyclone-911 board has a serial port that can be used to display
 * the results of the regression test. To that end, the MON960Device
 * provides the drivers for stdio to cause output to go through the
 * mon960_write call.
 *
 * To load this program, use Xmodem-1K to download the file.scof file
 * through the serial port on the Cyclone-board, then execute with the
 * "go" command at the mon960 console. The output will arrive at the
 * mon960 console.
 *
 * The program runs, then exits back to the monitor when it completes.
 */
# include  <ucr/ucr.h>
# include  <ucr/ISync.h>
# include  <stdio.h>
# include  <StdioDevice.h>
# include  <i960/mon960.h>
# include  <assert.h>

class MON960Device  : public StdioDevice {

    public:
      MON960Device()
      { flags_ = WRITEABLE; }

      int read(void*, size_t) { return 0; }

      void write(const void*str, size_t cnt)
      {
	    int out;
	    mon960_write(1, str, cnt, &out);
      }
};

class MON960DeviceType  : public StdioDeviceType {

    public:
      MON960DeviceType() : StdioDeviceType("console") { }

    private:
      MON960Device mon_;
      StdioDevice* open(const char*)
      { return &mon_; }

      void close(StdioDevice*)
      { }
};

MON960DeviceType mon960;

FILE*const stdout = fopen("console", "w");
void ucr_assert(const char*file, unsigned line)
{
      fprintf(stdout, "TEST FAILED at %s:%u\n", file, line);
      fflush(stdout);

      _exit(1);
}


extern int main_comm();

main()
{
      return main_comm();
}
