#ifndef __priv_H
#define __priv_H
/*
 * Copyright (c) 1998 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#if !defined(WINNT)
#ident "$Id: priv.h,v 1.8 1998/07/11 21:29:04 steve Exp $"
#endif

#if defined(WIN32)

# include  <windows.h>
# include  <io.h>

# define FILEDES HANDLE
# define close(fd) CloseHandle(fd)
static long ioctl(HANDLE fd, unsigned long cmd, long op)
{
      DWORD nout;
      long out;
      BOOL rc = DeviceIoControl(fd, cmd,
				&op, sizeof op,
				&out, sizeof out, &nout, 0);
      if (!rc) return -1;
      if (nout == 0) return 0;
      return out;
}

#else

# include  <fcntl.h>
# include  <unistd.h>

# define FILEDES int

#endif

/*
 * The following are the types and support for making the uCR channel
 * a first-class channel in the Tcl world. The ucr_channel_info
 * structure collects the data about the channel to pass around, then
 * the following functions use that data to manipulate the channel
 */

struct ucr_frame_info;
struct ucr_channel_info {
      FILEDES fd;

      unsigned long chan;
      int block_mode;
      Tcl_Interp*interp;
      Tcl_Channel tc;
      char *read_cmd;

      unsigned flags;

      struct ucr_frame_info*frames;
      Tcl_HashEntry*self;
};

struct ucr_frame_info {
      struct ucr_channel_info*cinfo;
      unsigned num;
      void* frame_base;
      unsigned long frame_size;

      struct ucr_frame_info*next;
      struct ucr_channel_info*ch;

      Tcl_Interp*interp;
      Tcl_Command cmd;
};

/* In the flags bitmask, if there is an event pending, set this so
   that I do not create gobs of useless events. */
# define EVENT_PENDING 2

extern int ucrOutputProc(ClientData cd, char*buf, int cnt, int*err);
extern int ucrInputProc(ClientData cd, char*buf, int cnt, int*err);
extern void ucrWatchProc(ClientData cd, int mask);

extern int ucr_open_channel(struct ucr_channel_info*info, const char*path,
			    Tcl_Interp*interp);

extern void ucr_read_proc(ClientData cd, int mask);

/*
 * This function does the operating system specific frame management
 * and mapping.
 */
extern int ucr_manage_frame(Tcl_Interp*interp, struct ucr_frame_info*finfo,
			    unsigned long frame_size);

/*
 * $Log: priv.h,v $
 * Revision 1.8  1998/07/11 21:29:04  steve
 *  Frames on multiple ISE boards.
 *
 * Revision 1.7  1998/06/10 04:09:12  steve
 *  redo the frame interface.
 *
 * Revision 1.6  1998/05/05 23:53:37  steve
 *  Software access to teh frame buffer.
 *
 * Revision 1.5  1998/04/17 22:29:24  steve
 *  Windows NT ioctls correct format.
 *
 * Revision 1.4  1998/04/15 15:50:00  steve
 *  Trace control.
 *
 * Revision 1.3  1998/03/23 19:56:09  steve
 *  Support for frames under NT,
 *  and non-blocking I/O under tcl/nt
 *
 * Revision 1.2  1998/03/12 23:00:25  steve
 *  Add frame commands.
 *
 * Revision 1.1  1998/03/12 21:32:28  steve
 *  divide the NT and linux implementataion up.
 *
 */
#endif
