/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: xact2img.cc,v 1.1 1997/02/05 05:25:06 steve Exp $"
/*
 * $Log: xact2img.cc,v $
 * Revision 1.1  1997/02/05 05:25:06  steve
 *  Add support for xact images.
 *
 */

/*
 * This file reads the Xact rawbits file and writes a binary image of
 * the important bits. The output image is in most-significant-bit
 * first, which corresponds to the order the XC4000 class loads the
 * bits into the Xilinx part.
 */

# include  <stdio.h>

/*
 * Read past the header stuff. When I get to a line that starts with a
 * '1' character, I know that I've arrived at the data.
 */
static void read_header(FILE*in)
{
      int prev = 0;
      for (;;) {
	    int c = fgetc(in);

	    if (c == EOF) return;

	    if ((prev == '\n') && (c == '1')) {
		  ungetc(c, in);
		  return;
	    }

	    prev = c;
      }
}

main(int argc, char*argv[])
{
      if (argc != 3) {
	    fprintf(stderr, "Usage: %s <in> <out>\n", argv[0]);
	    return -1;
      }

      FILE*in = fopen(argv[1], "r");
      if (in == 0) {
	    fprintf(stderr, "Cannot open %s for read.\n", argv[1]);
	    return 1;
      }

      FILE*out = fopen(argv[2], "wb");
      if (out == 0) {
	    fprintf(stderr, "Cannot open %s for writing.\n", argv[2]);
	    return 1;
      }

      read_header(in);

      unsigned char byte = 0;
      unsigned char nbyte = 0;
      for (int c = fgetc(in) ;  c != EOF ;  c = fgetc(in)) {
	    switch (c) {
		case '0':
		case '1':
		  byte = (byte<<1) | (c - '0');
		  nbyte += 1;
		  if (nbyte == 8) {
			fputc(byte, out);
			nbyte = 0;
			byte = 0;
		  }
		  break;
		case '\n':
		case '\r':
		  break;
		default:
		  fprintf(stderr, "Funky character `%c' in input.\n", c);
		  break;
	    }
      }

      if (nbyte > 0) {
	    byte <<= (8 - nbyte);
	    fputc(byte, out);
      }

      return 0;
}
